<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftSettings
{
	public static $optionPrefix = 'wpsseo_';
	public $settings = [];
	public $postTypes;

	public function __construct()
	{
		if (!is_admin()) {
			return;
		}

		register_activation_hook(WPSSEO_FILE, [$this, 'flushRewrites']);

		add_action('init', [$this, 'setupProperties'], 999);
		add_action('admin_menu', [$this, 'addSettingsMenus']);
		add_action('admin_init', [$this, 'registerSettings']);
		add_action('admin_init', [$this, 'purgeOption']);
		add_action('admin_init', [$this, 'warningActions']);
		add_action('admin_notices', [$this, 'showWarnings']);
		add_action('admin_notices', [$this, 'showUpdatedNotice']);
		add_action('admin_notices', [$this, 'otherSeoPluginsNotice']);

		add_action('admin_enqueue_scripts', [$this, 'enqueueScripts']);
		add_action('admin_footer', [$this, 'addWpsseoLoader']);

		add_action('after_setup_theme', [$this, 'saveSchemaOptions']);
	}

	public static function init()
	{
		return new self();
	}

	public function flushRewrites()
	{
		add_action('admin_init', function () {
			flush_rewrite_rules();
		}, 0);
	}

	public function enqueueScripts()
	{
		$wpsseoAssets = plugin_dir_url(WPSSEO_FILE) . 'assets';
		$cdnjsAssets = 'https://cdnjs.cloudflare.com/ajax/libs';
		$wpsseoId = WPSSEO_TEXT_DOMAIN . '-a';

		$titleAppend = '';
		if ((bool) get_option('wpsseo_append_to_title_tag', true)) {
			$titleAppend = ' &bull; ' . get_option('wpsseo_company_name', get_option('blogname'));
		}

		wp_enqueue_media();

		wp_enqueue_style($wpsseoId, $wpsseoAssets . '/css/admin.min.css', [], WPSSEO_VERSION);
		wp_enqueue_style('select2', $cdnjsAssets . '/select2/4.0.13/css/select2.min.css', [], WPSSEO_VERSION);

		$page = filter_input(INPUT_GET, 'page');
		if ($page === 'swift-seo-logs') {
			wp_enqueue_style('prism', $cdnjsAssets . '/prism/1.29.0/themes/prism-okaidia.min.css');
			wp_enqueue_script('prism', $cdnjsAssets . '/prism/1.29.0/prism.min.js', [], WPSSEO_VERSION, true);
		}

		wp_register_script($wpsseoId, $wpsseoAssets . '/js/admin.min.js', ['jquery'], WPSSEO_VERSION, true);
		wp_localize_script(
			$wpsseoId,
			'meta',
			[
				'imageTitle' => __('Choose or Upload Media', WPSSEO_TEXT_DOMAIN),
				'imageButton' => __('Use this image', WPSSEO_TEXT_DOMAIN),
				'titleMin' => WPSSEO_REC_TITLE_MIN_LENGTH,
				'titleMax' => WPSSEO_REC_TITLE_MAX_LENGTH,
				'descriptionMin' => WPSSEO_REC_DESC_MIN_LENGTH,
				'descriptionMax' => WPSSEO_REC_DESC_MAX_LENGTH,
				'badTitleMin' => WPSSEO_BAD_TITLE_MIN_LENGTH,
				'badTitleMax' => WPSSEO_BAD_TITLE_MAX_LENGTH,
				'badDescriptionMin' => WPSSEO_BAD_DESC_MIN_LENGTH,
				'badDescriptionMax' => WPSSEO_BAD_DESC_MAX_LENGTH,
				'emptyTitle' => __('Document Title Goes Here...', WPSSEO_TEXT_DOMAIN),
				'emptyDescription' => __('Meta Description Goes Here...', WPSSEO_TEXT_DOMAIN),
				'titleAppend' => $titleAppend,
				'blockPlaceholder' => __('Type / to choose a block'),
			]
		);

		$ajaxNonce = wp_create_nonce('wpsseo_ajax_nonce');
		wp_localize_script($wpsseoId, 'wpsseoAjax', [
			'url' => admin_url('admin-ajax.php'),
			'nonce' => $ajaxNonce,
		]);

		wp_enqueue_script('select2', $cdnjsAssets . '/select2/4.0.13/js/select2.min.js', ['jquery'], WPSSEO_VERSION, true);
		wp_enqueue_script('wp-color-picker');
		wp_enqueue_script($wpsseoId);
	}

	public function setupProperties()
	{
		if (!is_admin()) {
			return;
		}

		$this->postTypes = SwiftPostTypes::getPostTypes();
		$this->settings = $this->getSettings();
	}

	public function homePageSeoDescription()
	{
		echo '<p>' . __('This group is only for the dynamic home page (show latest posts). If using a static home page, the local SEO settings will overwrite these settings.', WPSSEO_TEXT_DOMAIN) . '</p>';
	}

	public function getSettings()
	{
		// Init array
		$settings = [];

		$category_example1 = '<code>' . home_url('/category/uncategorized/') . '</code>';
		$category_example2 = '<code>' . home_url('/uncategorized/') . '</code>';

		// General settings
		$settings['swift-seo-settings'] = [
			'full_title' => __('General SEO Settings', WPSSEO_TEXT_DOMAIN),
			'menu_title' => __('Swift SEO', WPSSEO_TEXT_DOMAIN),
			'tab_title' => __('Settings', WPSSEO_TEXT_DOMAIN),
			'groups' => [
				'general' => [
					'title' => __('General', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'enable_pro_preview',
							'type' => 'checkbox',
							'label' => __('Pro features', WPSSEO_TEXT_DOMAIN),
							'description' => __('Enable Pro features preview', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
					],
				],
				'webmaster' => [
					'title' => __('Webmaster Meta Settings', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'gsc_verification',
							'type' => 'text',
							'label' => __('Google Search Console Verification Code', WPSSEO_TEXT_DOMAIN),
							'description' => sprintf(__('%sGet the Google verification code%s', WPSSEO_TEXT_DOMAIN), '<a href="https://search.google.com/search-console/ownership?resource_id=' . urlencode(home_url()) . '" target="_blank">', '</a>'),
							'default' => '',
						],
						[
							'id' => 'bing_verification',
							'type' => 'text',
							'label' => __('Bing Webmaster Verification Code', WPSSEO_TEXT_DOMAIN),
							'description' => sprintf(__('%sGet the Bing verification code%s', WPSSEO_TEXT_DOMAIN), '<a href="https://www.bing.com/webmaster/home/addsite?addurl=' . urlencode(home_url()) . '" target="_blank">', '</a>'),
							'default' => '',
						],
						[
							'id' => 'yandex_verification',
							'type' => 'text',
							'label' => __('Yandex Webmaster Verification Code', WPSSEO_TEXT_DOMAIN),
							'description' => sprintf(__('%sGet the Yandex verification code%s', WPSSEO_TEXT_DOMAIN), '<a href="https://webmaster.yandex.com/sites/add/?hostName=' . urlencode(home_url()) . '" target="_blank">', '</a>'),
							'default' => '',
						],
						[
							'id' => 'baidu_verification',
							'type' => 'text',
							'label' => __('Baidu Search Resource Platform Code', WPSSEO_TEXT_DOMAIN),
							'description' => sprintf(__('%sGet the Baidu verification code%s', WPSSEO_TEXT_DOMAIN), '<a href="https://ziyuan.baidu.com/login/index?u=/site/siteadd" target="_blank">', '</a>'),
							'default' => '',
						],
					],
				],
				'home' => [
					'title' => __('Home Page SEO', WPSSEO_TEXT_DOMAIN),
					'description' => [$this, 'homePageSeoDescription'],
					'fields' => [
						[
							'id' => 'home_seo_title',
							'type' => 'text',
							'label' => __('Document title', WPSSEO_TEXT_DOMAIN),
							'description' => __('The document title sets the page title as seen in browsers and search engines.', WPSSEO_TEXT_DOMAIN),
							'default' => get_option('blogname'),
						],
						[
							'id' => 'home_seo_description',
							'type' => 'textarea',
							'size' => 'regular',
							'label' => __('Meta description', WPSSEO_TEXT_DOMAIN),
							'description' => __('Text entered in the Meta Description field is used as the short page description under the title on search engine results pages.', WPSSEO_TEXT_DOMAIN),
							'default' => get_bloginfo('description'),
						],
						[
							'id' => 'home_noindex',
							'type' => 'checkbox',
							'label' => __('Search results', WPSSEO_TEXT_DOMAIN),
							'description' => __('Hide from search results', WPSSEO_TEXT_DOMAIN),
							'default' => false,
						],
					],
				],
				'indexnow' => [
					'title' => __('IndexNow', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'indexnow_api_key',
							'type' => 'text',
							'label' => __('IndexNow API key', WPSSEO_TEXT_DOMAIN),
							'description' => __('For notifying search engines about new or updated content. This field disappears once it is saved.', WPSSEO_TEXT_DOMAIN),
							'default' => md5(uniqid('', true)),
						],
					],
				],
				'mods' => [
					'title' => __('WordPress Mods', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'remove_category_base',
							'type' => 'checkbox',
							'label' => __('Category base', WPSSEO_TEXT_DOMAIN),
							'description' => sprintf(__('Remove the category base from URLs. Eg. %s becomes %s', WPSSEO_TEXT_DOMAIN), $category_example1, $category_example2),
							'default' => false,
						],
						[
							'id' => 'notes',
							'type' => 'message',
							'label' => '',
							'description' => '<span style="font-weight: 500; color: #be0e0e">' . __('Removing the category base is generally not recommended, as it can lead to conflicts with page URLs and negatively impact your SEO.', WPSSEO_TEXT_DOMAIN) . '</span>',
						],
						[
							'id' => 'block_search_results',
							'type' => 'checkbox',
							'label' => __('Internal search', WPSSEO_TEXT_DOMAIN),
							'description' => __('Block internal search results from being crawled', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
						[
							'id' => 'remove_country_code',
							'type' => 'checkbox',
							'label' => __('Lang-attribute', WPSSEO_TEXT_DOMAIN),
							'description' => __('Remove country code from lang-attribute when not using multiple languages', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
						[
							'id' => 'remove_rss_feeds',
							'type' => 'checkbox',
							'label' => __('RSS feeds', WPSSEO_TEXT_DOMAIN),
							'description' => __('Remove default RSS feeds', WPSSEO_TEXT_DOMAIN),
							'default' => false,
						],
						[
							'id' => 'remove_generator_tags',
							'type' => 'checkbox',
							'label' => __('Generator tags', WPSSEO_TEXT_DOMAIN),
							'description' => __('Remove generator tags from the source code', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
						[
							'id' => 'remove_shortlink',
							'type' => 'checkbox',
							'label' => __('Shortlink', WPSSEO_TEXT_DOMAIN),
							'description' => __('Remove shortlink added by WP', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
						[
							'id' => 'disable_xmlrpc',
							'type' => 'checkbox',
							'label' => __('XML-RPC', WPSSEO_TEXT_DOMAIN),
							'description' => __('Disable XML-RPC and RSD link (recommended)', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
						[
							'id' => 'restrict_rest_api',
							'type' => 'checkbox',
							'label' => __('Restrict WP Rest API', WPSSEO_TEXT_DOMAIN),
							'description' => __('Restrict access to WooCommerce uploads and zip files through /wp-json/wp/v2/media endpoint (recommended)', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
					],
				],
			]
		];

		// Content settings
		$settings['swift-seo-content'] = [
			'full_title' => __('SEO Content Settings', WPSSEO_TEXT_DOMAIN),
			'menu_title' => __('Content', WPSSEO_TEXT_DOMAIN),
			'tab_title' => __('Content', WPSSEO_TEXT_DOMAIN),
			'groups' => [
				'content' => [
					'title' => __('Content formatting', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'append_to_title_tag',
							'type' => 'checkbox',
							'label' => __('Append to title', WPSSEO_TEXT_DOMAIN),
							'description' => __("Automatically add the company name to the website's title tag if the title length is within recommended limits.", WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
						[
							'id' => 'adult_rating',
							'type' => 'checkbox',
							'label' => __('Adult content', WPSSEO_TEXT_DOMAIN),
							'description' => __("Add an adult content rating to the website for Google SafeSearch.", WPSSEO_TEXT_DOMAIN),
							'default' => false,
						],
					],
				],
				'breadcrumbs' => [
					'title' => __('Breadcrumbs', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'use_breadcrumbs',
							'type' => 'checkbox',
							'label' => __('Use breadcrumbs', WPSSEO_TEXT_DOMAIN),
							'description' => __('Activate breadcrumbs shortcode and PHP function, and automatically include it in Schema markup.', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
						[
							'id' => 'breadcrumbs_implementation',
							'type' => 'message',
							'label' => __('Instructions', WPSSEO_TEXT_DOMAIN),
							'description' => sprintf(
								__('You can add breadcrumbs to any content by using the shortcode: %s[swift-breadcrumbs]%s%sTo implement breadcrumbs in your templates, you can use this PHP snippet: %s', WPSSEO_TEXT_DOMAIN),
								'<code>',
								'</code>',
								'<br>',
								'<code>&lt;?php SwiftBreadcrumbs::render(); ?&gt;</code>'
							),
						],
					],
				],
				'sitemap' => [
					'title' => __('XML Sitemap', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'sitemap_post_limits',
							'type' => 'number',
							'label' => __('Sitemap limit', WPSSEO_TEXT_DOMAIN),
							'description' => __("Maximum number of entries per sitemap. Don't worry, we'll automatically split the sitemap for you and for search engines to understand. Google allows a maximum of 50,000 entries or a 50 MB file size, so ensure your sitemaps do not exceed these limits.", WPSSEO_TEXT_DOMAIN),
							'default' => 10000,
						],
						[
							'id' => 'sitemap_notes',
							'type' => 'message',
							'label' => '',
							'description' => '<span style="font-weight: 500">' . sprintf(__('View your sitemap at %s', WPSSEO_TEXT_DOMAIN), '<a href="' . home_url(SwiftSitemap::getSitemapName()) . '" target="_blank">' . home_url(SwiftSitemap::getSitemapName()) . '</a>') . '</span>',
						],
					],
				],
			]
		];

		// Visibility settings
		$settings['swift-seo-visibility'] = [
			'full_title' => __('Visibility Settings', WPSSEO_TEXT_DOMAIN),
			'menu_title' => __('Visibility', WPSSEO_TEXT_DOMAIN),
			'tab_title' => __('Visibility', WPSSEO_TEXT_DOMAIN),
			'groups' => [
				[
					'title' => __('Indexing Settings', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'exclude_noindex_from_search',
							'type' => 'checkbox',
							'label' => __('Exclude noindexed posts', WPSSEO_TEXT_DOMAIN),
							'description' => __('Exclude noindexed posts from the local site search', WPSSEO_TEXT_DOMAIN),
							'default' => false,
						],
						[
							'id' => 'allow_pdf_indexing',
							'type' => 'checkbox',
							'label' => __('PDF index', WPSSEO_TEXT_DOMAIN),
							'description' => __('Allow indexing of PDF files', WPSSEO_TEXT_DOMAIN),
							'default' => false,
						],
						[
							'id' => 'noindex_empty_terms',
							'type' => 'checkbox',
							'label' => __('Taxonomy Terms', WPSSEO_TEXT_DOMAIN),
							'description' => __('Automatically noindex empty terms', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
					],
				],
				[
					'title' => __('Global noindex', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'noindex_post_types',
							'type' => 'checkbox_group',
							'label' => __('Apply noindex to post types', WPSSEO_TEXT_DOMAIN),
							'options' => $this->postTypes,
							'default' => ['attachment'],
							'inline' => false,
						],
						[
							'id' => 'noindex_taxonomies',
							'type' => 'checkbox_group',
							'label' => __('Apply noindex to taxonomies', WPSSEO_TEXT_DOMAIN),
							'options' => SwiftTaxonomies::getTaxonomies(),
							'default' => ['post_tag'],
							'inline' => false,
						],
						[
							'id' => 'noindex_archives',
							'type' => 'checkbox_group',
							'label' => __('Apply noindex to archives', WPSSEO_TEXT_DOMAIN),
							'options' => [
								'author' => __('Author', WPSSEO_TEXT_DOMAIN),
								'date' => __('Date', WPSSEO_TEXT_DOMAIN),
								'search' => __('Search', WPSSEO_TEXT_DOMAIN),
							],
							'default' => ['date', 'search'],
							'inline' => true,
						],
					],
				],
				[
					'title' => __('Global nofollow', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'nofollow_post_types',
							'type' => 'checkbox_group',
							'label' => __('Apply nofollow to post types', WPSSEO_TEXT_DOMAIN),
							'options' => $this->postTypes,
							'default' => '',
							'inline' => false,
						],
						[
							'id' => 'nofollow_taxonomies',
							'type' => 'checkbox_group',
							'label' => __('Apply nofollow to taxonomies', WPSSEO_TEXT_DOMAIN),
							'options' => SwiftTaxonomies::getTaxonomies(),
							'default' => '',
							'inline' => false,
						],
						[
							'id' => 'nofollow_archives',
							'type' => 'checkbox_group',
							'label' => __('Apply nofollow to archives', WPSSEO_TEXT_DOMAIN),
							'options' => [
								'author' => __('Author', WPSSEO_TEXT_DOMAIN),
								'date' => __('Date', WPSSEO_TEXT_DOMAIN),
								'search' => __('Search', WPSSEO_TEXT_DOMAIN),
							],
							'default' => [],
							'inline' => true,
						],
					],
				],
			]
		];

		if (!WPSSEO_IS_PUBLIC) {
			$settings['swift-seo-visibility']['groups'][1]['description'] = function () {
				echo '<p><strong class="wpsseo-visibility-warning no-margin">' . sprintf(__('Your website is set to block search engine indexing. To adjust these settings, please modify the search engine visibility %shere%s.', WPSSEO_TEXT_DOMAIN), '<a href="' . admin_url('options-reading.php') . '">', '</a>') . '</strong></p>';
			};
		}

		// Schema settings
		$settings['swift-seo-schema'] = [
			'full_title' => __('Company/Schema Settings', WPSSEO_TEXT_DOMAIN),
			'menu_title' => __('Schema', WPSSEO_TEXT_DOMAIN),
			'tab_title' => __('Company/Schema', WPSSEO_TEXT_DOMAIN),
			'groups' => [
				[
					'title' => __('Company Contact Information', WPSSEO_TEXT_DOMAIN),
					'description' => function () {
						_e('These settings have multiple uses, so please fill them out completely.', WPSSEO_TEXT_DOMAIN);
					},
					'fields' => [
						[
							'id' => 'company_name',
							'type' => 'text',
							'label' => __('Company Name', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'default' => get_option('blogname'),
						],
						[
							'id' => 'company_co_address',
							'type' => 'text',
							'label' => __('C/O Address', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'default' => '',
						],
						[
							'id' => 'company_address',
							'type' => 'text',
							'label' => __('Address', WPSSEO_TEXT_DOMAIN),
							'description' => __('Physical address of the company', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
						[
							'id' => 'company_address_2',
							'type' => 'text',
							'label' => __('Address Line 2', WPSSEO_TEXT_DOMAIN),
							'description' => __('Optional', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
						[
							'id' => 'company_zip',
							'type' => 'text',
							'label' => __('Zip/postal code', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'default' => '',
						],
						[
							'id' => 'company_city',
							'type' => 'text',
							'label' => __('City', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'default' => '',
						],
						[
							'id' => 'company_country',
							'type' => 'select',
							'label' => __('Country', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'options' => array_replace([0 => __('Select country...', WPSSEO_TEXT_DOMAIN)], SwiftUtilities::countriesByContinent()),
							'default' => '',
						],
						[
							'id' => 'company_email',
							'type' => 'text',
							'label' => __('Email address', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'default' => get_option('admin_email'),
						],
						[
							'id' => 'company_phone',
							'type' => 'text',
							'label' => __('Phone number', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'default' => '',
						],
						[
							'id' => 'company_vat',
							'type' => 'text',
							'label' => __('VAT number', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'default' => '',
						],
					],
				],
				[
					'title' => __('Company Information', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'company_image',
							'type' => 'image',
							'label' => __('Company image', WPSSEO_TEXT_DOMAIN),
							'description' => __('Representational image of the company. If no image is provided, the website logo will be used.', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
						[
							'id' => 'price_range',
							'type' => 'text',
							'label' => __('Price range', WPSSEO_TEXT_DOMAIN),
							'description' => __("Enter price range for the company's services/products", WPSSEO_TEXT_DOMAIN),
							'default' => '0',
						],
						[
							'id' => 'currencies_accepted',
							'type' => 'text',
							'label' => __('Currencies accepted', WPSSEO_TEXT_DOMAIN),
							'description' => __('For example: USD, EUR, BTC', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
						[
							'id' => 'payment_accepted',
							'type' => 'text',
							'label' => __('Payment accepted', WPSSEO_TEXT_DOMAIN),
							'description' => __('For example: Cash, Credit Card, Cryptocurrency', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
					],
				],
				[
					'title' => __('Organization Type', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'organization_type',
							'type' => 'select',
							'label' => __('Organization Type', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'include_sub_keys' => true,
							'options' => $this->getOrganizationTypes(),
							'default' => '',
						],
					],
				],
				[
					'title' => __('Navigation', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'primary_navigation',
							'type' => 'select',
							'label' => __('Primary navigation', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'options' => $this->listNavMenus(),
							'default' => '',
						],
					],
				],
			],
		];

		// Social settings
		$settings['swift-seo-social'] = [
			'full_title' => __('Social Settings', WPSSEO_TEXT_DOMAIN),
			'menu_title' => __('Social', WPSSEO_TEXT_DOMAIN),
			'tab_title' => __('Social', WPSSEO_TEXT_DOMAIN),
			'groups' => [
				[
					'title' => __('Social Profiles', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'extended_social_meta',
							'type' => 'checkbox',
							'label' => __('Extended social meta', WPSSEO_TEXT_DOMAIN),
							'description' => __('Enable extended social meta tags. Eg. author, article reading time, product price etc.', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
						[
							'id' => 'social_profiles',
							'type' => 'text_group',
							'label' => __('Social profiles for schema markup', WPSSEO_TEXT_DOMAIN),
							'description' => '',
							'options' => [
								'facebook' => 'Facebook URL',
								'linkedin' => 'Linkedin URL',
								'youtube' => 'Youtube URL',
								'twitter' => 'Twitter URL',
								'github' => 'Github URL',
								'stackoverflow' => 'Stack Overflow URL',
								'skype' => 'Skype URL',
								'snapchat' => 'Snapchat URL',
								'twitch' => 'Twitch URL',
								'steam' => 'Steam URL',
								'pinterest' => 'Pinterest URL',
								'vimeo' => 'Vimeo URL',
								'instagram' => 'Instagram URL',
							],
						],
					],
				],
				[
					'title' => __('Analytics', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'facebook_app_id',
							'type' => 'text',
							'label' => __('Facebook app ID', WPSSEO_TEXT_DOMAIN),
							'description' => __('The ID of a linked Facebook application', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
						[
							'id' => 'twitter_site_id',
							'type' => 'text',
							'label' => __('Twitter site', WPSSEO_TEXT_DOMAIN),
							'description' => __('The user ID of a related Twitter site. Eg. "@YourSite"', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
						[
							'id' => 'pinterest_verification',
							'type' => 'text',
							'label' => __('Pinterest Analytics verification code', WPSSEO_TEXT_DOMAIN),
							'description' => sprintf(__('%sGet the Pinterest verification code%s', WPSSEO_TEXT_DOMAIN), '<a href="https://analytics.pinterest.com/" target="_blank">', '</a>'),
							'default' => '',
						],
					],
				],
				[
					'title' => __('Social Image', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'default_social_image',
							'type' => 'image',
							'label' => __('Default social image', WPSSEO_TEXT_DOMAIN),
							'description' => __('This image will be used as social image if no other suitable image was found.', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
					],
				],
			],
		];

		// Script settings
		$settings['swift-seo-scripts'] = [
			'full_title' => __('Scripts', WPSSEO_TEXT_DOMAIN),
			'menu_title' => __('Scripts', WPSSEO_TEXT_DOMAIN),
			'tab_title' => __('Scripts', WPSSEO_TEXT_DOMAIN),
			'groups' => [
				[
					'title' => __('Header scripts', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'analytics',
							'type' => 'textarea',
							'size' => 'large',
							'label' => __('Header scripts', WPSSEO_TEXT_DOMAIN),
							'description' => __('Used for, among other things, analytics script', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
						[
							'id' => 'analytics_admin_users',
							'type' => 'checkbox',
							'label' => __('Header scripts for admins', WPSSEO_TEXT_DOMAIN),
							'description' => __('Load header scripts for admin users', WPSSEO_TEXT_DOMAIN),
							'default' => false,
						],
					],
				],
				[
					'title' => __('Body scripts', WPSSEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'body_scripts',
							'type' => 'textarea',
							'size' => 'large',
							'label' => __('Body scripts', WPSSEO_TEXT_DOMAIN),
							'description' => __('These scripts will load right before the ending body tag.', WPSSEO_TEXT_DOMAIN),
							'default' => '',
						],
						[
							'id' => 'body_scripts_admin_users',
							'type' => 'checkbox',
							'label' => __('Body scripts for admins', WPSSEO_TEXT_DOMAIN),
							'description' => __('Load body scripts for admin users', WPSSEO_TEXT_DOMAIN),
							'default' => true,
						],
					]
				]
			],
		];

		$indexnowApiKey = get_option('wpsseo_indexnow_api_key');
		if (!empty($indexnowApiKey)) {
			$serverRoot = self::getServerRoot();
			$indexnowVerificationFile = "$serverRoot/$indexnowApiKey.txt";

			if (!file_exists($indexnowVerificationFile)) {
				@touch($indexnowVerificationFile);
				@chmod($indexnowVerificationFile, 0664);
				@file_put_contents($indexnowVerificationFile, $indexnowApiKey);
			}

			unset($settings['swift-seo-settings']['groups']['indexnow']);
		}

		$frontPageId = get_option('page_on_front');
		if (!empty($frontPageId)) {
			unset($settings['swift-seo-settings']['groups']['home']);
		}

		$breadcrumbsActive = (bool) get_option('wpsseo_use_breadcrumbs', true);
		if (!$breadcrumbsActive) {
			unset($settings['swift-seo-settings']['groups']['breadcrumbs']['fields'][1]);
		}

		return $settings;
	}

	public function registerSettings()
	{
		$this->registerSettingsFields($this->settings);
	}

	public static function getServerRoot($trailingSlash = false)
	{
		$docRoot = filter_input(INPUT_SERVER, 'DOCUMENT_ROOT');

		if (empty($docRoot)) {
			if (wp_is_writable(ABSPATH)) {
				$docRoot = ABSPATH;
			} else {
				$docRoot = dirname(WP_CONTENT_DIR);
			}
		}

		$docRoot = htmlspecialchars(rtrim($docRoot, '/'));

		if ($trailingSlash) {
			$docRoot .= '/';
		}

		return $docRoot;
	}

	public function listNavMenus()
	{
		$menuArray = [];

		$menus = wp_get_nav_menus();

		if (!empty($menus)) {
			foreach ($menus as $menu) {
				$menuName = $menu->name;
				$menuSlug = $menu->slug;

				$menuArray[$menuSlug] = $menuName;
			}
		}

		return $menuArray;
	}

	private function getOrganizationTypes()
	{
		return [
			'Organization' => 'Organization',
			'LocalBusiness' => 'LocalBusiness',
		];
	}

	public function addSettingsMenus()
	{
		$counter = 1;
		$mainSettings = '';

		foreach ($this->settings as $key => $data) {
			$fullTitle = $data['full_title'];
			$menuTitle = $data['menu_title'];
			$subMenuTitle = $data['tab_title'] ?? $menuTitle;

			if ($counter === 1) {
				$mainSettings = $key;
				$iconUrl = plugin_dir_url(WPSSEO_FILE) . 'assets/img/swift.svg';
				$menuIcon = 'data:image/svg+xml;base64,' . base64_encode(SwiftUtilities::getUrlContent($iconUrl));
				add_menu_page($fullTitle, $menuTitle, 'manage_options', $key, [$this, 'renderSettingsPage'], $menuIcon, 96);
				add_submenu_page($key, $fullTitle, $subMenuTitle, 'manage_options', $key, [$this, 'renderSettingsPage']);
			} else {
				add_submenu_page($mainSettings, $fullTitle, $menuTitle, 'manage_options', $key, [$this, 'renderSettingsPage']);
			}

			++$counter;
		}
	}

	public function renderSettingsPage()
	{
		$currentPage = htmlspecialchars(filter_input(INPUT_GET, 'page'));

		if (!empty($this->settings[$currentPage])) {
			$settings = $this->settings[$currentPage];

			echo '<div class="wrap">
				<h1>' . $settings['full_title'] . '</h1>
				<form method="post" action="options.php" novalidate="novalidate">';

			$this->addTabs();

			settings_fields($currentPage);
			do_settings_sections($currentPage);
			submit_button();

			echo '</form>
			</div>';
		}
	}

	private function addTabs()
	{
		echo '<h2 class="nav-tab-wrapper wpsseo-tabs">' . "\n";

		$currentPage = htmlspecialchars(filter_input(INPUT_GET, 'page'));
		foreach ($this->settings as $key => $section) {
			if (empty($section['tab_title'])) {
				continue;
			}

			$url = admin_url('/admin.php?page=' . $key);
			$title = $section['tab_title'];
			$active = $currentPage == $key ? true : false;

			$class = 'nav-tab';
			if (!empty($active)) {
				$class .= ' nav-tab-active';
			}

			echo '<a href="' . esc_attr($url) . '" class="' . esc_attr($class) . '">' . esc_html($title) . '</a>' . "\n";
		}

		echo '</h2>' . "\n";
	}

	public function registerSettingsFields($settings)
	{
		foreach ($settings as $location => $data) {
			foreach ($data['groups'] as $key => $group) {
				$groupTitle = $group['title'];
				$groupId = sanitize_title($groupTitle);
				$descriptionCallback = $group['description'] ?? '__return_empty_string';
				$fields = $group['fields'];

				$groupClass = '';
				if (!empty($group['toggle_by'])) {
					$groupClass = ' hidden toggle-by-' . $group['toggle_by'];
				}

				add_settings_section($groupId, $groupTitle, $descriptionCallback, $location, ['before_section' => '<div class="postbox wpsseo wpsseo-settings' . $groupClass . '"><div class="inside">', 'after_section' => '</div></div>']);

				foreach ($fields as $field) {
					if (!is_array($field)) {
						continue;
					}

					if (empty($field['id'])) {
						$id = md5((!empty($field['description']) ? $field['description'] : $field['type']) . microtime(true));
						$field['id'] = self::$optionPrefix . $id;
					} else {
						$field['id'] = self::$optionPrefix . $field['id'];
					}

					if (!empty($field['toggle_by'])) {
						$field['class'] = 'hidden toggle-by-' . $field['toggle_by'];
					}

					register_setting($location, $field['id']);
					add_settings_field($field['id'], ($field['label'] ?? ''), [$this, 'renderSettingsField'], $location, $groupId, $field);
				}
			}
		}
	}

	public function renderSettingsField($field)
	{
		switch ($field['type']) {
			case 'checkbox':
				$dataAtts = '';
				if (!empty($field['toggle_target'])) {
					$dataAtts .= ' data-toggle-target=".toggle-by-' . esc_attr($field['toggle_target']) . '" data-toggle-value="1"';
				}

				echo '<fieldset>';

				$default = get_option($field['id'], $field['default']);
				$disabled = (bool) ($field['disabled'] ?? false);

				echo '<label for="' . esc_attr($field['id']) . '" class="wpsseo-check-default"><input name="' . esc_attr($field['id']) . '" id="' . esc_attr($field['id']) . '" type="checkbox" value="1"' . $dataAtts . (!empty($default) ? ' checked' : '') . (!empty($disabled) ? ' disabled' : '') . '><span class="check"></span> ' . $field['description'] . '</label>';

				echo '</fieldset>';

				break;
			case 'checkbox_group':
				echo '<fieldset class="checkbox-group' . (!empty($field['inline']) ? ' inline' : '') . '">';

				if (!empty($field['options'])) {
					$default = get_option($field['id'], $field['default']);

					if (!is_array($default)) {
						$default = [$default];
					}

					$i = 0;
					foreach ($field['options'] as $key => $label) {
						echo '<label for="' . esc_attr($field['id']) . '-' . ++$i . '" class="wpsseo-check-default"><input name="' . esc_attr($field['id']) . '[' . $key . ']" id="' . esc_attr($field['id']) . '-' . $i . '" type="checkbox" value="1"' . (in_array($key, $default) || !empty($default[$key]) ? ' checked' : '') . '><span class="check"></span> ' . $label . '</label>';
					}
				}

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				echo '</fieldset>';

				break;
			case 'image':
				$imageId = get_option($field['id']);
				$imageObject = '';

				if (!empty($imageId)) {
					$imageObject = wp_get_attachment_image($imageId, 'medium');
				}

				echo '<input name="' . esc_attr($field['id']) . '" id="' . esc_attr($field['id']) . '" type="hidden" value="' . esc_attr($imageId) . '">
				<style>#' . esc_attr($field['id']) . '-preview .image img{margin-bottom: 10px;}</style>
				<div id="' . esc_attr($field['id']) . '-preview"><div class="image">' . $imageObject . '</div></div>
				<input class="wpsseo-upload-image wpsseo-button" type="button" data-id="' . esc_attr($field['id']) . '" value="' . esc_attr__('Select image', WPSSEO_TEXT_DOMAIN) . '">
				<input class="remove-image wpsseo-button" type="button" data-id="' . esc_attr($field['id']) . '" value="' . esc_attr__('Remove image', WPSSEO_TEXT_DOMAIN) . '"' . (empty($imageId) ? ' style="display: none;"' : '') . '>';

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				break;
			case 'select':
				$withKeys = (bool) ($field['with_keys'] ?? false);
				$dataAtts = '';
				if (!empty($field['toggle_target'])) {
					$dataAtts .= ' data-toggle-target=".toggle-by-' . esc_attr($field['toggle_target']) . '"';
				}
				if (!empty($field['toggle_value'])) {
					$dataAtts .= ' data-toggle-value="' . esc_attr($field['toggle_value']) . '"';
				}

				echo '<select name="' . esc_attr($field['id']) . '" id="' . esc_attr($field['id']) . '" class="regular-text styled-select"' . $dataAtts . '>';

				if (!empty($field['options'])) {
					$default = (string) get_option($field['id'], $field['default']);
					$includeKey = $field['include_sub_keys'] ?? false;

					foreach ($field['options'] as $key => $value) {
						if (!$withKeys && is_numeric($key)) {
							$key = $value;
						}

						if (is_array($value)) {
							echo '<optgroup label="' . esc_attr($key) . '">';

							if ($includeKey) {
								echo '<option value="' . esc_attr($key) . '"' . ((string) $key === $default ? ' selected' : '') . '>' . $key . '</option>';
							}

							foreach ($value as $subKey => $subValue) {
								if (!$withKeys && is_numeric($subKey)) {
									$subKey = $subValue;
								}

								if (is_array($subValue)) {
									foreach ($subValue as $subSubKey => $subSubValue) {
										if (!$withKeys && is_numeric($subSubKey)) {
											$subSubKey = $subSubValue;
										}

										$subSubValue = str_repeat('&nbsp;', 3) . $subSubValue;

										echo '<option value="' . esc_attr($subSubKey) . '"' . ((string) $subSubKey === $default ? ' selected' : '') . '>' . $subSubValue . '</option>';
									}
								} else {
									echo '<option value="' . esc_attr($subKey) . '"' . ((string) $subKey === $default ? ' selected' : '') . '>' . $subValue . '</option>';
								}
							}

							echo '</optgroup>';
						} else {
							echo '<option value="' . esc_attr($key) . '"' . ((string) $key === $default ? ' selected' : '') . '>' . $value . '</option>';
						}
					}
				}

				echo '</select>';

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				break;
			case 'select_multi':
				echo '<select name="' . esc_attr($field['id']) . '[]" id="' . esc_attr($field['id']) . '" class="regular-text styled-select" multiple>';

				if (!empty($field['options'])) {
					$default = get_option($field['id'], $field['default']);

					if (!is_array($default)) {
						$default = [$default];
					}

					foreach ($field['options'] as $key => $value) {
						echo '<option value="' . esc_attr($key) . '"' . (in_array($key, $default) ? ' selected' : '') . '>' . esc_html($value) . '</option>';
					}
				}

				echo '</select>';

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				break;
			case 'radio':
				echo '<fieldset>';

				if (!empty($field['options'])) {
					$default = (string) get_option($field['id'], $field['default']);

					$i = 0;
					foreach ($field['options'] as $key => $value) {
						$dataAtts = '';
						if (!empty($field['toggle_target'])) {
							$dataAtts .= ' data-toggle-target=".toggle-by-' . esc_attr($field['toggle_target']) . '" data-toggle-value="' . esc_attr($key) . '"';
						}

						echo '<label for="' . esc_attr($field['id']) . '-' . ++$i . '" class="wpsseo-radio-default"><input name="' . esc_attr($field['id']) . '" id="' . $field['id'] . '-' . $i . '" type="' . esc_attr($field['type']) . '" value="' . esc_attr($key) . '"' . $dataAtts . ((string) $key === $default ? ' checked' : '') . '><span class="check"></span> ' . $value . '</label>';
					}
				}

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				echo '</fieldset>';

				break;
			case 'color':
				$default = get_option($field['id'], $field['default']);

				echo '<input name="' . esc_attr($field['id']) . '" id="' . esc_attr($field['id']) . '" type="text" value="' . esc_attr($default) . '" data-default-color="' . esc_attr($default) . '" class="regular-text color-picker">';

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				break;
			case 'textarea':
				$default = get_option($field['id'], $field['default']);
				$size = $field['size'] ?? 'regular';
				$class = $size . '-text';
				$rows = $size === 'large' ? 12 : 6;

				echo '<textarea name="' . esc_attr($field['id']) . '" id="' . esc_attr($field['id']) . '" rows="' . $rows . '" class="' . $class . ' code">' . esc_html($default) . '</textarea>';

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				break;
			case 'number':
				$default = get_option($field['id'], $field['default']);

				$min = $field['min'] ?? '0';
				$max = $field['max'] ?? null;
				$step = $field['step'] ?? '1';

				echo '<input name="' . esc_attr($field['id']) . '" id="' . esc_attr($field['id']) . '" type="' . esc_attr($field['type']) . '" value="' . esc_attr($default) . '" min="' . esc_attr($min) . '"' . (!empty($max) ? ' max="' . esc_attr($max) . '"' : '') . ' step="' . esc_attr($step) . '" class="regular-text">';

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				break;
			case 'password':
				$default = esc_attr(get_option($field['id']));
				$placeholder = $field['placeholder'] ?? '';
				$purgeButton = (bool) ($field['purge_button'] ?? true);
				$purgeButtonText = $field['purge_button_text'] ?? __('Purge', WPSSEO_TEXT_DOMAIN);
				$confirmDialog = $field['confirm_dialog'] ?? __('Are you sure?', WPSSEO_TEXT_DOMAIN);

				$params = '';
				if (!empty($default)) {
					$default = str_repeat('*', mb_strlen($default));
					$params = ' readonly="readonly" style="pointer-events: none"';
				}

				echo '<div class="wpsseo-password-wrapper">';
				echo '<input name="' . esc_attr($field['id']) . '" id="' . esc_attr($field['id']) . '" type="' . esc_attr($field['type']) . '" value="' . esc_attr($default) . '" placeholder="' . esc_attr($placeholder) . '" class="regular-text" autocomplete="off"' . $params . '>';

				if (!empty($default) && $purgeButton) {
					echo '<a href="#" class="wpsseo-button purge" data-confirm="' . $confirmDialog . '">' . $purgeButtonText . '</a>';
				}

				echo '</div>';

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				break;
			case 'editor':
				$default = get_option($field['id']);

				wp_editor($default, esc_attr($field['id']));

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				break;
			case 'message':
				if (!empty($field['description'])) {
					echo '<div class="description">' . $field['description'] . '</div>';
				}

				break;
			case 'text_group':
				echo '<fieldset>';

				if (!empty($field['options'])) {
					$saved = get_option($field['id'], []);

					$i = 0;
					foreach ($field['options'] as $key => $value) {
						$default = $saved[$key] ?? '';

						if (++$i > 1) {
							echo '<br>';
						}

						echo '<label for="' . esc_attr($field['id']) . '-' . $i . '" class="text-group">' . $value . '</label>';
						echo '<input name="' . esc_attr($field['id']) . '[' . $key . ']" id="' . esc_attr($field['id']) . '-' . $i . '" type="text" value="' . esc_attr($default) . '" class="regular-text">';
					}
				}

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}

				echo '</fieldset>';

				break;
			default:
				$default = get_option($field['id'], $field['default']);

				echo '<input name="' . esc_attr($field['id']) . '" id="' . esc_attr($field['id']) . '" type="' . esc_attr($field['type']) . '" value="' . esc_attr($default) . '" class="regular-text">';

				if (!empty($field['description'])) {
					echo '<p class="description" id="' . esc_attr($field['id']) . '-description">' . $field['description'] . '</p>';
				}
		}
	}

	public function saveSchemaOptions()
	{
		if (!is_admin()) {
			return;
		}

		$page = filter_input(INPUT_GET, 'page');
		if ($page !== 'swift-seo-schema' && $page !== 'swift-seo-social') {
			return;
		}

		$updated = filter_input(INPUT_GET, 'settings-updated');
		if (empty($updated)) {
			return;
		}

		if (empty($this->settings)) {
			$this->settings = $this->getSettings();
		}

		$schemaGroups = $this->settings['swift-seo-schema']['groups'];
		$socialGroups = $this->settings['swift-seo-social']['groups'];

		$options = [];
		$optionNames = [];

		foreach ($schemaGroups as $group) {
			$fields = $group['fields'];
			foreach ($fields as $field) {
				$optionNames[] = self::$optionPrefix . $field['id'];
			}
		}

		foreach ($socialGroups as $group) {
			$fields = $group['fields'];
			foreach ($fields as $field) {
				$optionNames[] = self::$optionPrefix . $field['id'];
			}
		}

		foreach ($optionNames as $optionName) {
			$key = str_replace(self::$optionPrefix, '', $optionName);
			$options[$key] = get_option($optionName);
		}

		update_option('wpsseo_schema_data', $options);

		// Clear All Meta Tags Transients
		global $wpdb;
		$wpdb->query("DELETE FROM `{$wpdb->prefix}options` WHERE `option_name` LIKE '_transient_wpsseo_meta_tags%' OR `option_name` LIKE '_transient_timeout_wpsseo_meta_tags%'");
	}

	public function purgeOption()
	{
		$action = filter_input(INPUT_POST, 'action');

		if ($action !== 'purgeOption') {
			return;
		}

		if (!check_ajax_referer('wpsseo_ajax_nonce', 'nonce')) {
			wp_send_json_error(['message' => __('Page has expired, please reload and try again.', WPSSEO_TEXT_DOMAIN)]);
		}

		$option = htmlspecialchars(filter_input(INPUT_POST, 'option'));
		$deleted = (bool) delete_option($option);
		$message = !$deleted ? __('We were unable to remove the value.', WPSSEO_TEXT_DOMAIN) : '';

		if ($deleted) {
			wp_send_json_success(['message' => $message]);
		} else {
			wp_send_json_error(['message' => $message]);
		}
	}

	public function showWarnings()
	{
		$serverRoot = self::getServerRoot();
		$adminUrl = admin_url();
		$errorClass = 'notice notice-error swift-notice';
		$warningClass = 'notice notice-warning swift-notice';

		$sitemapName = SwiftSitemap::getSitemapName();
		$sitemapFile = "$serverRoot/$sitemapName";
		$sitemapXmlError = get_option('sitemap_xml_error');
		$ignoreSitemapXml = get_option('ignore_sitemap_xml');

		$robotsFile = "$serverRoot/robots.txt";
		$robotsTxtError = get_option('robots_txt_error');
		$ignoreRobotsTxt = get_option('ignore_robots_txt');

		$ignoreNoindex = get_option('ignore_noindex');

		if (empty($ignoreSitemapXml)) {
			if (file_exists($sitemapFile) && empty($sitemapXmlError)) {
				$ignoreSitemapUrl = add_query_arg([
					'page' => 'swift-seo-settings',
					'action' => 'ignore-sitemap-xml',
					'_wpnonce' => wp_create_nonce('ignore-sitemap-xml'),
				], $adminUrl);

				$deleteSitemapUrl = add_query_arg([
					'page' => 'swift-seo-settings',
					'action' => 'delete-sitemap-xml',
					'_wpnonce' => wp_create_nonce('delete-sitemap-xml'),
				], $adminUrl);

				$message = sprintf(__('%sWarning!%s We detected a %s file in the root of your website. Our dynamic sitemap will not work until you delete this file.%sRemember to always make a backup before deleting.', WPSSEO_TEXT_DOMAIN), '<strong style="text-decoration: underline">', '</strong>', '<code style="border: 1px solid #ccc;border-radius: 3px;">' . $sitemapName . '</code>', '<br>');
				$message .= '<div style="margin-top: 10px">';
				$message .= '<a href="' . $deleteSitemapUrl . '" class="wpsseo-button small" style="margin-left: 0;" onclick="return confirm(\'' . __('Are you sure you want to permanently delete the file? This action cannot be reverted.', WPSSEO_TEXT_DOMAIN) . '\')">' . __('Delete File', WPSSEO_TEXT_DOMAIN) . '</a>';
				$message .= '<a href="' . $ignoreSitemapUrl . '" class="wpsseo-button small ignore" style="margin-left: 8px;">' . __("Remove this warning", WPSSEO_TEXT_DOMAIN) . '</a>';
				$message .= '</div>';

				printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($warningClass), $message);
			} elseif (!empty($sitemapXmlError)) {
				printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($errorClass), $sitemapXmlError);
			} elseif (!file_exists($sitemapFile)) {
				delete_option('sitemap_xml_error');
			}
		}

		if (empty($ignoreRobotsTxt)) {
			if (file_exists($robotsFile) && empty($robotsTxtError)) {
				$ignoreRobotsUrl = add_query_arg([
					'page' => 'swift-seo-settings',
					'action' => 'ignore-robots-txt',
					'_wpnonce' => wp_create_nonce('ignore-robots-txt'),
				], $adminUrl);

				$deleteRobotsUrl = add_query_arg([
					'page' => 'swift-seo-settings',
					'action' => 'delete-robots-txt',
					'_wpnonce' => wp_create_nonce('delete-robots-txt'),
				], $adminUrl);

				$message = sprintf(__('%sWarning!%s We detected a %s file in the root of your website. Our dynamic robots.txt will not work until you delete this file.%sRemember to always make a backup before deleting.', WPSSEO_TEXT_DOMAIN), '<strong style="text-decoration: underline">', '</strong>', '<code style="border: 1px solid #ccc;border-radius: 3px;">robots.txt</code>', '<br>');
				$message .= '<div style="margin-top: 10px">';
				$message .= '<a href="' . $deleteRobotsUrl . '" class="wpsseo-button small" style="margin-left: 0;" onclick="return confirm(\'' . __('Are you sure you want to permanently delete the file? This action cannot be reverted.', WPSSEO_TEXT_DOMAIN) . '\')">' . __('Delete File', WPSSEO_TEXT_DOMAIN) . '</a>';
				$message .= '<a href="' . $ignoreRobotsUrl . '" class="wpsseo-button small ignore" style="margin-left: 8px;">' . __("Remove this warning", WPSSEO_TEXT_DOMAIN) . '</a>';
				$message .= '</div>';

				printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($warningClass), $message);
			} elseif (!empty($robotsTxtError)) {
				printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($warningClass), $robotsTxtError);
			} elseif (!file_exists($robotsFile)) {
				delete_option('robots_txt_error');
			}
		}

		if (empty($ignoreNoindex) && !WPSSEO_IS_PUBLIC) {
			$publishUrl = add_query_arg([
				'page' => 'swift-seo-settings',
				'action' => 'publish-website',
				'_wpnonce' => wp_create_nonce('publish-website'),
			], $adminUrl);

			$ignoreIndexUrl = add_query_arg([
				'page' => 'swift-seo-settings',
				'action' => 'ignore-noindex',
				'_wpnonce' => wp_create_nonce('ignore-noindex'),
			], $adminUrl);

			$message = sprintf(__('%sWarning!%s Your settings may prevent search engines from indexing your site.', WPSSEO_TEXT_DOMAIN), '<strong style="text-decoration: underline">', '</strong>');
			$message .= '<div class="wpsseo-publish-button-group">';
			$message .= '<a href="' . $publishUrl . '" class="wpsseo-button">' . __('Publish website', WPSSEO_TEXT_DOMAIN) . '</a>';
			$message .= '<a href="' . $ignoreIndexUrl . '" class="wpsseo-button ignore">' . __("Dismiss", WPSSEO_TEXT_DOMAIN) . '</a>';
			$message .= '</div>';

			printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($errorClass), $message);
		}
	}

	public function warningActions()
	{
		$action = filter_input(INPUT_GET, 'action');
		$serverRoot = self::getServerRoot();

		if ($action === 'ignore-sitemap-xml') {
			$nonce = filter_input(INPUT_GET, '_wpnonce');
			$referrer = htmlspecialchars(filter_input(INPUT_SERVER, 'HTTP_REFERER'));

			if (!wp_verify_nonce($nonce, $action)) {
				wp_die(__('Verification failed.', WPSSEO_TEXT_DOMAIN));
			}

			update_option('ignore_sitemap_xml', true);

			wp_redirect($referrer);
		}

		if ($action === 'delete-sitemap-xml') {
			$nonce = filter_input(INPUT_GET, '_wpnonce');
			$referrer = htmlspecialchars(filter_input(INPUT_SERVER, 'HTTP_REFERER'));

			if (!wp_verify_nonce($nonce, $action)) {
				wp_die(__('Verification failed.', WPSSEO_TEXT_DOMAIN));
			}

			$sitemapName = SwiftSitemap::getSitemapName();
			$sitemapFile = "$serverRoot/$sitemapName";
			$deleted = unlink($sitemapFile);

			if (!$deleted) {
				update_option('sitemap_xml_error', sprintf(__('We were unable to remove the sitemap.xml file. Please remove it manually from %s', WPSSEO_TEXT_DOMAIN), '<code style="border: 1px solid #ccc;border-radius: 3px;">' . $sitemapFile . '</code>'));
			} else {
				delete_option('sitemap_xml_error');
				delete_option('ignore_sitemap_xml');
			}

			wp_redirect($referrer);
		}

		if ($action === 'ignore-robots-txt') {
			$nonce = filter_input(INPUT_GET, '_wpnonce');
			$referrer = htmlspecialchars(filter_input(INPUT_SERVER, 'HTTP_REFERER'));

			if (!wp_verify_nonce($nonce, $action)) {
				wp_die(__('Verification failed.', WPSSEO_TEXT_DOMAIN));
			}

			update_option('ignore_robots_txt', true);

			wp_redirect($referrer);
		}

		if ($action === 'delete-robots-txt') {
			$nonce = filter_input(INPUT_GET, '_wpnonce');
			$referrer = htmlspecialchars(filter_input(INPUT_SERVER, 'HTTP_REFERER'));

			if (!wp_verify_nonce($nonce, $action)) {
				wp_die(__('Verification failed.', WPSSEO_TEXT_DOMAIN));
			}

			$robotsFile = "$serverRoot/robots.txt";
			$deleted = unlink($robotsFile);

			if (!$deleted) {
				update_option('robots_txt_error', sprintf(__('We were unable to remove the robots.txt file. Please remove it manually from %s', WPSSEO_TEXT_DOMAIN), '<code style="border: 1px solid #ccc;border-radius: 3px;">' . $robotsFile . '</code>'));
			} else {
				delete_option('robots_txt_error');
				delete_option('ignore_robots_txt');
			}

			wp_redirect($referrer);
		}

		if ($action === 'ignore-noindex') {
			$nonce = filter_input(INPUT_GET, '_wpnonce');
			$referrer = htmlspecialchars(filter_input(INPUT_SERVER, 'HTTP_REFERER'));

			if (!wp_verify_nonce($nonce, $action)) {
				wp_die(__('Verification failed.', WPSSEO_TEXT_DOMAIN));
			}

			update_option('ignore_noindex', true);

			wp_redirect($referrer);
		}

		if ($action === 'publish-website') {
			$nonce = filter_input(INPUT_GET, '_wpnonce');
			$referrer = htmlspecialchars(filter_input(INPUT_SERVER, 'HTTP_REFERER'));

			if (!wp_verify_nonce($nonce, $action)) {
				wp_die(__('Verification failed.', WPSSEO_TEXT_DOMAIN));
			}

			update_option('blog_public', true);
			update_option('swift_theme_environment', 'production');

			wp_redirect($referrer);
		}
	}

	public function showUpdatedNotice()
	{
		if (!is_admin()) {
			return;
		}

		$settingsPages = array_keys($this->settings);
		$page = filter_input(INPUT_GET, 'page');
		if (!in_array($page, $settingsPages)) {
			return;
		}

		$updated = filter_input(INPUT_GET, 'settings-updated');
		if (empty($updated)) {
			return;
		}

		if (get_option('wpsseo_disable_xmlrpc', true)) {
			update_option('default_ping_status', '');
		}

		global $wpdb;
		$wpdb->query($wpdb->prepare("DELETE FROM `{$wpdb->prefix}options` WHERE `option_name` LIKE %s OR `option_name` LIKE %s", "_transient_wpsseo_meta_tags_%", "_transient_timeout_wpsseo_meta_tags_%"));

		delete_transient('wpsseo_plugin_update');
		flush_rewrite_rules();

		$class = 'notice notice-success is-dismissable swift-notice';
		printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), __('Settings saved', WPSSEO_TEXT_DOMAIN));
	}

	public function otherSeoPluginsNotice()
	{
		$seoPlugin = self::detectSeoPlugins();
		if (!$seoPlugin) {
			return;
		}

		$class = 'notice notice-warning swift-notice';
		$message = __("%s We detected that you have the %s plugin installed. To avoid conflicts, we highly recommend disabling it.", WPSSEO_TEXT_DOMAIN);

		printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), sprintf($message, '<span class="dashicons dashicons-warning" style="color: #72aee6"></span>', '<strong>' . $seoPlugin . '</strong>'));
	}

	public static function detectSeoPlugins()
	{
		if (!function_exists('is_plugin_active')) {
			include_once(ABSPATH . 'wp-admin/includes/plugin.php');
		}

		$seoPlugins = [
			'yoast' => [
				'path' => 'wordpress-seo/wp-seo.php',
				'name' => 'Yoast SEO'
			],
			'rankmath' => [
				'path' => 'seo-by-rank-math/rank-math.php',
				'name' => 'Rank Math'
			],
			'seopress' => [
				'path' => 'wp-seopress/seopress.php',
				'name' => 'SEOPress'
			],
			'aioseo' => [
				'path' => 'all-in-one-seo-pack/all_in_one_seo_pack.php',
				'name' => 'All in One SEO Pack'
			],
			'slim' => [
				'path' => 'slim-seo/slim-seo.php',
				'name' => 'Slim SEO'
			],
		];

		foreach ($seoPlugins as $slug => $pluginInfo) {
			if (is_plugin_active($pluginInfo['path'])) {
				return $pluginInfo['name'];
			}
		}

		return false;
	}

	public function addWpsseoLoader()
	{
		echo '<div class="wpsseo-loader hidden"><div class="wpsseo-spinner"></div><span class="wpsseo-loader-content">' . __("We're working - Please wait...", WPSSEO_TEXT_DOMAIN) . '</span></div>';
	}
}
