<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftBreadcrumbs
{
	public static $args = [
		'wrap_start' => '<div class="breadcrumbs">',
		'wrap_end' => '</div>',
		'separator' => ' <span class="sep">&rsaquo;</span> ',
		'item_wrap_start' => '',
		'item_wrap_end' => '',
	];

	public function __construct()
	{
		if (is_admin()) {
			return;
		}

		$breadcrumbsActive = (bool) get_option('wpsseo_use_breadcrumbs', true);

		if (!$breadcrumbsActive) {
			return;
		}

		add_shortcode('swift-breadcrumbs', [__CLASS__, 'render']);
	}

	public static function init()
	{
		return new self();
	}

	private static function setArgs()
	{
		self::$args = apply_filters('wpsseo_breadcrumb_args', self::$args);
	}

	public static function getSerpBreadcrumb($url)
	{
		if (!filter_var($url, FILTER_VALIDATE_URL)) {
			return [$url];
		}

		$urlInfo = wp_parse_url($url);
		$domain = $urlInfo['scheme'] . '://' . $urlInfo['host'];

		$breadcrumb = [$domain];

		if (empty($urlInfo['path'])) {
			return $breadcrumb;
		}

		$path = trim($urlInfo['path'], '/');
		$parts = array_filter(explode('/', $path));

		foreach ($parts as $part) {
			$breadcrumb[] = $part;
		}

		return $breadcrumb;
	}

	public static function getBreadcrumbsStructure()
	{
		$homePageId = (int) get_option('page_on_front', 0);
		$homePageLabel = $fallbackLabel = apply_filters('wp_title', _x('Home', 'Breadcrumbs Home', WPSSEO_TEXT_DOMAIN));

		if (!empty($homePageId)) {
			$homePageLabel = get_post_meta($homePageId, '_breadcrumb_title', true);

			if (empty($homePageLabel)) {
				$homePageLabel = $fallbackLabel;
			}
		}

		$breadcrumbs = [
			[
				'url' => esc_url(home_url()),
				'label' => esc_html($homePageLabel),
				'add_link' => true,
			]
		];

		if (is_404()) {
			$breadcrumbs[] = [
				'url' => esc_url(home_url()),
				'label' => __('404 - Page not found', WPSSEO_TEXT_DOMAIN),
				'add_link' => false,
			];
		} elseif (is_search()) {
			$query = get_search_query();
			$label = !empty($query) ? sprintf(__('Search for %s', WPSSEO_TEXT_DOMAIN), esc_html($query)) : __('Empty search', WPSSEO_TEXT_DOMAIN);

			$breadcrumbs[] = [
				'url' => esc_url(home_url('?s=' . $query)),
				'label' => $label,
				'add_link' => false,
			];
		} elseif (is_author()) {
			$authorId = get_the_author_meta('ID');
			$label = get_user_meta($authorId, '_breadcrumb_title', true) ?: get_the_author_meta('display_name');
			$url = get_author_posts_url($authorId);

			$breadcrumbs[] = [
				'url' => esc_url($url),
				'label' => esc_html($label),
				'add_link' => false,
			];
		} elseif (SwiftMetaTags::isDynamicFrontPage() || SwiftMetaTags::isFrontPage()) {
			$breadcrumbs = [];
		} elseif (SwiftMetaTags::isHome()) {
			$homePageId = (int) get_option('page_for_posts', 0);

			if (!empty($homePageId)) {
				$label = get_post_meta($homePageId, '_breadcrumb_title', true) ?: get_the_title($homePageId);
				$url = get_permalink($homePageId);

				$breadcrumbs[] = [
					'url' => esc_url($url),
					'label' => esc_html($label),
					'add_link' => false,
				];
			}
		} elseif (WPSSEO_IS_WOOCOMMERCE && is_shop()) {
			$shopPageId = (int) get_option('woocommerce_shop_page_id', 0);

			if (!empty($shopPageId)) {
				$label = get_post_meta($shopPageId, '_breadcrumb_title', true) ?: get_the_title($shopPageId);
				$url = get_permalink($shopPageId);

				$breadcrumbs[] = [
					'url' => esc_url($url),
					'label' => esc_html($label),
					'add_link' => false,
				];
			}
		} elseif (is_singular(array_keys(SwiftPostTypes::getPostTypes()))) {
			$post = get_post();

			if ($post->post_type === 'post' || $post->post_type === 'product') {
				if ($post->post_type === 'post') {
					$permalinkStructure = get_option('permalink_structure');
					$homePageId = (int) get_option('page_for_posts', 0);
					$taxonomy = 'category';
				} else {
					$permalinkStructure = get_option('woocommerce_permalinks');
					$permalinkStructure = $permalinkStructure['product_base'] ?? '/products/%product_cat%/';
					$homePageId = (int) get_option('woocommerce_shop_page_id', 0);
					$taxonomy = 'product_cat';
				}

				if (!empty($homePageId)) {
					$homeLabel = get_post_meta($homePageId, '_breadcrumb_title', true) ?: get_the_title($homePageId);
					$homeUrl = get_permalink($homePageId);

					$breadcrumbs[] = [
						'url' => esc_url($homeUrl),
						'label' => esc_html($homeLabel),
						'add_link' => true,
					];
				}

				if (stristr($permalinkStructure, "%{$taxonomy}%") !== false) {
					$categories = get_the_category($post->ID);

					if (!empty($categories)) {
						$primaryCategory = get_post_meta($post->ID, '_primary_category', 'true');

						if (empty($primaryCategory) || count($categories) < 2) {
							$firstCategory = reset($categories);
							$primaryCategory = $firstCategory->term_id;
						}

						$ancestors = (array) get_ancestors($primaryCategory, $taxonomy, 'taxonomy');
						$ancestors = array_reverse($ancestors);
						$ancestors[] = $primaryCategory;

						foreach ($ancestors as $categoryId) {
							$category = get_term($categoryId, $taxonomy);

							if (empty($category)) {
								continue;
							}

							$termLabel = get_term_meta($categoryId, '_breadcrumb_title', true) ?: $category->name;
							$termUrl = get_term_link($category);

							if (is_wp_error($termUrl)) {
								continue;
							}

							$breadcrumbs[] = [
								'url' => esc_url($termUrl),
								'label' => esc_html($termLabel),
								'add_link' => true,
							];
						}
					}
				}

				$label = get_post_meta($post->ID, '_breadcrumb_title', true) ?: $post->post_title;
				$url = get_permalink($post->ID);

				$breadcrumbs[] = [
					'url' => esc_url($url),
					'label' => esc_html($label),
					'add_link' => false,
				];
			} else {
				$ancestors = (array) get_post_ancestors($post);

				if (!empty($ancestors)) {
					$ancestors = array_reverse($ancestors);

					foreach ($ancestors as $post_id) {
						$label = get_post_meta($post_id, '_breadcrumb_title', true) ?: get_the_title($post_id);
						$url = get_permalink($post_id);

						$breadcrumbs[] = [
							'url' => esc_url($url),
							'label' => esc_html($label),
							'add_link' => true,
						];
					}
				}

				$label = get_post_meta($post->ID, '_breadcrumb_title', true) ?: get_the_title($post->ID);
				$url = get_permalink($post->ID);

				$breadcrumbs[] = [
					'url' => esc_url($url),
					'label' => esc_html($label),
					'add_link' => false,
				];
			}
		} elseif ($term = get_queried_object()) {
			if (is_a($term, 'WP_Term')) {
				$ancestors = (array) get_ancestors($term->term_id, $term->taxonomy, 'taxonomy');

				if ($term->taxonomy === 'category' || $term->taxonomy === 'post_tag') {
					$homePageId = (int) get_option('page_for_posts', 0);

					if (!empty($homePageId)) {
						$label = get_post_meta($homePageId, '_breadcrumb_title', true) ?: get_the_title($homePageId);
						$url = get_permalink($homePageId);

						$breadcrumbs[] = [
							'url' => esc_url($url),
							'label' => esc_html($label),
							'add_link' => true,
						];
					}
				}

				if ($term->taxonomy === 'product_cat' || $term->taxonomy === 'product_tag') {
					$homePageId = (int) get_option('woocommerce_shop_page_id', 0);

					if (!empty($homePageId)) {
						$label = get_post_meta($homePageId, '_breadcrumb_title', true) ?: get_the_title($homePageId);
						$url = get_permalink($homePageId);

						$breadcrumbs[] = [
							'url' => esc_url($url),
							'label' => esc_html($label),
							'add_link' => true,
						];
					}
				}

				if (!empty($ancestors)) {
					$ancestors = array_reverse($ancestors);

					foreach ($ancestors as $categoryId) {
						$category = get_term($categoryId, $term->taxonomy);
						$label = get_term_meta($categoryId, '_breadcrumb_title', true) ?: $category->name;
						$url = get_term_link($category);

						if (is_wp_error($url)) {
							continue;
						}

						$breadcrumbs[] = [
							'url' => esc_url($url),
							'label' => esc_html($label),
							'add_link' => true,
						];
					}
				}

				$label = get_term_meta($term->term_id, '_breadcrumb_title', true) ?: $term->name;
				$url = get_term_link($term);

				$breadcrumbs[] = [
					'url' => esc_url($url),
					'label' => esc_html($label),
					'add_link' => false,
				];
			}
		}

		return apply_filters('wpsseo_breadcrumb_structure', $breadcrumbs);
	}

	private static function getBreadcrumbsLinks()
	{
		self::setArgs();

		$breadcrumbsLinks = self::getBreadcrumbsStructure();
		$returnLinks = [];

		if (!empty($breadcrumbsLinks)) {
			foreach ($breadcrumbsLinks as $link) {
				if (!empty($link['add_link'])) {
					$returnLinks[] = self::$args['item_wrap_start'] . '<a href="' . $link['url'] . '">' . $link['label'] . '</a>' . self::$args['item_wrap_end'];

					continue;
				}

				$returnLinks[] = self::$args['item_wrap_start'] . $link['label'] . self::$args['item_wrap_end'];
			}
		}

		return apply_filters('wpsseo_breadcrumb_links', $returnLinks);
	}

	public static function render()
	{
		if (is_admin()) {
			return;
		}

		self::setArgs();

		$breadcrumbsLinks = self::getBreadcrumbsLinks();

		if (!empty($breadcrumbsLinks)) {
			return self::$args['wrap_start'] . implode(self::$args['separator'], $breadcrumbsLinks) . self::$args['wrap_end'];
		}

		return '';
	}
}
