<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftMetaBoxes
{
	public function __construct()
	{
		if (!is_admin()) {
			return;
		}

		add_action('wp_ajax_reloadSeoHealth', [$this, 'reloadSeoHealth']);
		add_action('load-post.php', [$this, 'initMetabox']);
		add_action('load-post-new.php', [$this, 'initMetabox']);
		add_action('wp_ajax_generateAi', [$this, 'generateAi']);
	}

	public static function init()
	{
		return new self();
	}

	public function initMetabox()
	{
		add_action('add_meta_boxes', [$this, 'addMetabox']);
		add_action('save_post', [$this, 'saveMetabox'], 10, 2);
	}

	public function addMetabox()
	{
		$postTypes = array_keys(SwiftPostTypes::getPostTypes());

		add_meta_box(
			'wpsseo-metabox',
			__('Search Engine Optimization - Swift SEO', WPSSEO_TEXT_DOMAIN),
			[$this, 'renderSeoMetabox'],
			$postTypes,
			'normal',
			'high'
		);

		$post = get_post();
		$priority = 'high';

		if (!empty($post->post_type) && $post->post_type === 'product') {
			$priority = 'default';
		}

		add_meta_box(
			'wpsseo_primary_category',
			__('Select Primary Category', WPSSEO_TEXT_DOMAIN),
			[$this, 'primaryCategoryMetabox'],
			['post', 'product'],
			'side',
			$priority
		);
	}

	public function generateAi()
	{
		if (!check_ajax_referer('wpsseo_ajax_nonce', 'nonce')) {
			wp_send_json_error([
				'output' => __('Page has expired, please reload and try again.', WPSSEO_TEXT_DOMAIN),
			]);
		}

		$input = trim(filter_input(INPUT_POST, 'input'));
		$type = htmlspecialchars(filter_input(INPUT_POST, 'type')); // title, description

		if (empty(trim($input))) {
			wp_send_json_error([
				'input' => $input,
				'type' => $type,
				'output' => sprintf(__('We found no content to generate %s from.', WPSSEO_TEXT_DOMAIN), $type),
			]);
		}

		$input = htmlspecialchars($input);
		$output = (new SwiftOpenAi())->metaByInput($input, $type);

		if (!empty($output) && !empty($output['success'])) {
			$text = $output['text'] ?? '';
			$debugging = $output['debugging'] ?? '';
			$tokens = $output['tokens'] ?? 0;

			wp_send_json_success([
				'input' => $input,
				'type' => $type,
				'output' => $text,
				'debugging' => $debugging,
				'tokens' => $tokens,
				'token_info' => sprintf(__('This request used %s tokens.', WPSSEO_TEXT_DOMAIN), $tokens),
			]);
		}

		$text = $output['text'] ?? '';
		$debugging = $output['debugging'] ?? '';
		$tokens = $output['tokens'] ?? 0;

		wp_send_json_error([
			'input' => $input,
			'type' => $type,
			'output' => $text,
			'debugging' => $debugging,
			'tokens' => $tokens,
			'token_info' => sprintf(__('This request used %s tokens.', WPSSEO_TEXT_DOMAIN), $tokens),
		]);
	}

	public function primaryCategoryMetabox($post)
	{
		$postId = $post->ID;
		$categoryIds = [];

		if ($post->post_type === 'product') {
			$chosenCategories = get_the_terms($postId, 'product_cat');
			$allCategories = get_terms(['taxonomy' => 'product_cat', 'hide_empty' => false]);
		} else {
			$chosenCategories = get_the_category($postId);
			$allCategories = get_terms(['taxonomy' => 'category', 'hide_empty' => false]);
		}

		foreach ($chosenCategories as $category) {
			$categoryIds[] = $category->term_id;
		}

		$primaryCategory = get_post_meta($postId, '_primary_category', true);
		$defaultCategory = get_option('default_category');

		if (empty($primaryCategory)) {
			$primaryCategory = $defaultCategory;
		}

		if (empty($categoryIds) && !empty($defaultCategory)) {
			$categoryIds[] = $defaultCategory;
		}

		echo '<label for="primary_category">' . __('Select Primary Category', WPSSEO_TEXT_DOMAIN) . '</label>
		<fieldset>
			<select name="primary_category" id="primary_category" style="width: calc(100% - 35px);" data-default="' . esc_attr($defaultCategory) . '">';

		foreach ($allCategories as $category) {
			echo '<option value="' . esc_attr($category->term_id) . '"' . (!empty($categoryIds) && !in_array($category->term_id, $categoryIds) ? ' disabled' : '') . ($primaryCategory == $category->term_id ? ' selected' : '') . '>' . esc_html($category->name) . '</option>';
		}

		echo '</select>
		</fieldset>';
	}

	public function reloadSeoHealth()
	{
		if (!check_ajax_referer('wpsseo_ajax_nonce', 'nonce')) {
			wp_send_json_error(['message' => __('Page has expired, please reload and try again.', WPSSEO_TEXT_DOMAIN)]);
		}

		$postId = intval($_POST['post_id'] ?? 0);

		try {
			$post = get_post($postId);
			$seoAnalysis = SwiftSeoAnalysis::getSignals($post);

			wp_send_json_success([
				'html' => $seoAnalysis['html'],
				'indicator' => $seoAnalysis['indicator'],
			]);
		} catch (Exception $e) {
			wp_send_json_error([
				'message' => $e->getMessage()
			]);
		}
	}

	// Static function to make it available for all content types
	public static function seoMetabox($object, $type = 'post')
	{
		if ($type === 'term') {
			$objectId = $object->term_id;
			$objectMeta = SwiftMeta::term($objectId);
			$metaTitle = $objectMeta['doctitle'] ?? '';
			$metaDescription = $objectMeta['description'] ?? '';
			$noindex = (bool) ($objectMeta['noindex'] ?? false);
			$nofollow = (bool) ($objectMeta['nofollow'] ?? false);
			$permalink = get_term_link($object);
			$objectMeta['_meta_tags_cache'] = SwiftMeta::term($objectId, '_meta_tags_cache');
		} elseif ($type === 'author') {
			$objectId = $object->ID;
			$objectMeta = SwiftMeta::user($objectId);
			$metaTitle = $objectMeta['doctitle'] ?? '';
			$metaDescription = $objectMeta['meta_description'] ?? '';
			$noindex = (bool) ($objectMeta['noindex'] ?? false);
			$nofollow = (bool) ($objectMeta['nofollow'] ?? false);
			$permalink = get_author_posts_url($objectId);
			$objectMeta['_meta_tags_cache'] = SwiftMeta::user($objectId, '_meta_tags_cache');
		} elseif ($type === 'post') {
			$objectId = $object->ID;
			$objectMeta = SwiftMeta::post($objectId);
			$metaTitle = $objectMeta['_genesis_title'] ?? '';
			$metaDescription = $objectMeta['_genesis_description'] ?? '';
			$noindex = (bool) ($objectMeta['_genesis_noindex'] ?? false);
			$nofollow = (bool) ($objectMeta['_genesis_nofollow'] ?? false);
			$permalink = get_permalink($object);
			$objectMeta['_meta_tags_cache'] = SwiftMeta::post($objectId, '_meta_tags_cache');

			if (strpos($permalink, '?p=') !== false) {
				list($permalink, $postName) = get_sample_permalink($objectId);
				$permalink = str_replace('%postname%', $postName, $permalink);
			}
		} else {
			return '';
		}

		$enableProPreview = (bool) get_option('wpsseo_enable_pro_preview', true);

		$autoTitle = $objectMeta['_auto_title'] ?? '';
		$autoDescription = $objectMeta['_auto_description'] ?? '';
		$canonical = $objectMeta['_genesis_canonical_uri'] ?? '';
		$metaTagsCache = (bool) ($objectMeta['_meta_tags_cache'] === '' ? true : (bool) $objectMeta['_meta_tags_cache']);
		$primaryKeyword = $objectMeta['_primary_keyword'] ?? '';
		$secondaryKeywords = $objectMeta['_secondary_keywords'] ?? '';
		$socialImageId = (int) ($objectMeta['_social_image_id'] ?? 0);
		$socialImageObject = '';
		$breadcrumbTitle = $objectMeta['_breadcrumb_title'] ?? '';
		$headerScripts = $objectMeta['_genesis_scripts'] ?? '';
		$bodyScripts = $objectMeta['_genesis_scripts_body'] ?? '';

		if (!empty($socialImageId)) {
			$socialImageObject = wp_get_attachment_image($socialImageId, 'medium');
		}

		$companyName = get_option('wpsseo_company_name', get_option('blogname'));
		$appendToTitleTag = (bool) get_option('wpsseo_append_to_title_tag', true);
		$append = '';

		if ($appendToTitleTag) {
			$append = ' &bull; ' . $companyName;
		}

		wp_nonce_field('save_seo_meta', 'save_seo_meta');

		$visibilityWarning = '';
		if (!WPSSEO_IS_PUBLIC) {
			$visibilityWarning = '<p><strong class="wpsseo-visibility-warning">' . sprintf(__('Your settings may prevent search engines from indexing your site. To adjust these settings, please modify the search engine visibility %shere%s.', WPSSEO_TEXT_DOMAIN), '<a href="' . admin_url('options-reading.php') . '">', '</a>') . '</strong></p>';
		}

		$html = '<p class="nav-tab-wrapper wpsseo-tabs meta' . ($type === 'post' ? ' post' : '') . '">
			<a class="nav-tab nav-tab-active" href="#seo-settings">' . __('SEO Settings', WPSSEO_TEXT_DOMAIN) . '</a>
			<a class="nav-tab" href="#page-scripts">' . __('Scripts', WPSSEO_TEXT_DOMAIN) . '</a>
			' . ($enableProPreview ? '<a class="nav-tab" href="#seo-analysis">' . __('SEO Health', WPSSEO_TEXT_DOMAIN) . ' <span class="wpsseo-indicator audit0"></span></a>' : '') . '
		</p>';

		$showBreadcrumbs = (bool) get_option('wpsseo_use_breadcrumbs', true);
		$generateTitleFrom = get_option('wpsseo_ai_title_source', 'headline');
		$generateDescriptionFrom = get_option('wpsseo_ai_description_source', 'title');

		/** Meta Title Start **/
		$appendLength = mb_strlen($append);

		$metaTitleValue = $metaTitle;
		$metaTitle = !empty($metaTitle) ? $metaTitle : $autoTitle;
		$totalLength = mb_strlen((string) $metaTitle) + $appendLength;

		if (!empty($metaTitle) && $totalLength <= WPSSEO_REC_TITLE_MAX_LENGTH && strstr($metaTitle, $companyName) === false) {
			$metaTitle .= $append;
		}

		$titleChars = mb_strlen($metaTitle);
		$titleClass = 'audit1';

		if ($titleChars < WPSSEO_REC_TITLE_MIN_LENGTH) {
			$titleClass = 'audit2';

			if ($titleChars < WPSSEO_REC_TITLE_MIN_LENGTH - 10) {
				$titleClass = 'audit3';
			}
		}

		if ($titleChars > WPSSEO_REC_TITLE_MAX_LENGTH) {
			$titleClass = 'audit2';

			if ($titleChars > WPSSEO_REC_TITLE_MAX_LENGTH + 10) {
				$titleClass = 'audit3';
			}
		}

		$serpTitle = $metaTitle;
		if (mb_strlen($serpTitle) > WPSSEO_REC_TITLE_MAX_LENGTH) {
			$serpTitle = substr($metaTitle, 0, WPSSEO_REC_TITLE_MAX_LENGTH - 3) . '&hellip;';
		}

		if (empty($serpTitle)) {
			$serpTitle = __('Document Title Goes Here...', WPSSEO_TEXT_DOMAIN);
		}
		/** Meta Title End **/

		/** Meta Description Start **/
		$descriptionChars = mb_strlen($metaDescription);
		$descriptionClass = 'audit1';

		if ($descriptionChars < WPSSEO_REC_DESC_MIN_LENGTH) {
			$descriptionClass = 'audit2';

			if ($descriptionChars < WPSSEO_REC_DESC_MIN_LENGTH - 20) {
				$descriptionClass = 'audit3';
			}
		}

		if ($descriptionChars > WPSSEO_REC_DESC_MAX_LENGTH) {
			$descriptionClass = 'audit2';

			if ($descriptionChars > WPSSEO_REC_DESC_MAX_LENGTH + 10) {
				$descriptionClass = 'audit3';
			}
		}

		$serpDescription = $metaDescription;
		if (mb_strlen($serpDescription) > WPSSEO_REC_DESC_MAX_LENGTH) {
			$serpDescription = substr($metaDescription, 0, WPSSEO_REC_DESC_MAX_LENGTH - 3) . '&hellip;';
		}

		if (empty($serpDescription) || $serpDescription === __('Type / to choose a block')) {
			$serpDescription = esc_html($autoDescription ?: __('Meta Description Goes Here...', WPSSEO_TEXT_DOMAIN));
		}
		/** Meta Description End **/

		$html .= '<div id="seo-settings" class="tab-content">'; // Start tab content

		$icon = '<span class="dashicons dashicons-admin-site"></span>';
		$faviconUrl = get_site_icon_url(32);

		if (!empty($faviconUrl)) {
			$icon = '<div class="thumbnail"><img src="' . $faviconUrl . '" alt=""></div>';
		}

		$breadcrumb = implode(' <span class="separator">&#10095;</span> ', SwiftBreadcrumbs::getSerpBreadcrumb($permalink));

		$html .= '<div class="wpsseo-serp-preview-wrap">'; // Start serp preview wrap
		$html .= '<div id="wpsseo-serp-preview">
			<div class="header">
				<div class="image">' . $icon . '</div>
				<div class="info">
					<div class="domain">' . str_replace('www.', '', wp_parse_url($permalink, PHP_URL_HOST)) . '</div>
					<div class="breadcrumb">' . $breadcrumb . '</div>
				</div>
			</div>
			<div class="title">' . $serpTitle . '</div>
			<div class="description">' . $serpDescription . '</div>
		</div>';
		$html .= '</div>'; // End serp preview wrap

		$html .= '<div class="wpsseo-metabox-wrap">'; // Start metabox wrap
		$html .= '<h3 class="wpsseo-section-headline">' . __('General SEO Settings', WPSSEO_TEXT_DOMAIN) . '</h3>';
		$html .= '<table class="form-table seo-table">
			<tbody>
				<tr>
					<th scope="row">
						<label for="wpsseo_title">' . __('Document Title', WPSSEO_TEXT_DOMAIN) . '<span class="screen-reader-text"> ' . sprintf(__('%s Tag.', WPSSEO_TEXT_DOMAIN), '&lt;title&gt;') . ' </span></label>
						<p class="character-counter hide-if-no-js">' . __('Characters Used:', WPSSEO_TEXT_DOMAIN) . ' <span id="wpsseo_title_chars" class="' . $titleClass . '">' . $titleChars . '</span></p>
					</th>
					<td>
						<p class="wpsseo-input-wrapper">
							<input class="widefat" type="text" name="wpsseo_title" id="wpsseo_title" value="' . esc_attr($metaTitleValue) . '" placeholder="' . esc_attr($metaTitle ?: __('Document Title Goes Here...', WPSSEO_TEXT_DOMAIN)) . '">
							' . (!empty($enableProPreview) ? '<a href="#" class="wpsseo-button ai-optimize" data-id="wpsseo_title" data-type="title" data-source="' . esc_attr($generateTitleFrom) . '" data-tooltip="' . __('(Only Pro) Generate an SEO optimized document title.', WPSSEO_TEXT_DOMAIN) . '">&#x1FA84; ' . esc_attr_x('AI', 'Generate button', WPSSEO_TEXT_DOMAIN) . '</a>' : '') . '
						</p>
						<p class="description">' . sprintf(__('The Document Title sets the page title as seen in browsers and search engines. Recommended length is %s-%s characters.', WPSSEO_TEXT_DOMAIN), WPSSEO_REC_TITLE_MIN_LENGTH, WPSSEO_REC_TITLE_MAX_LENGTH) . '</p>
					</td>
				</tr>

				<tr>
					<th scope="row">
						<label for="wpsseo_description">' . __('Meta Description', WPSSEO_TEXT_DOMAIN) . '<span class="screen-reader-text"> &lt;meta name="description"&gt;.</span></label>
						<p class="character-counter hide-if-no-js">' . __('Characters Used:', WPSSEO_TEXT_DOMAIN) . ' <span id="wpsseo_description_chars" class="' . $descriptionClass . '">' . $descriptionChars . '</span></p>
					</th>
					<td>
						<p class="wpsseo-input-wrapper">
							<textarea class="widefat" name="wpsseo_description" id="wpsseo_description" rows="4" placeholder="' . esc_attr($autoDescription ?: __('Meta Description Goes Here...', WPSSEO_TEXT_DOMAIN)) . '">' . esc_html($metaDescription) . '</textarea>
							' . (!empty($enableProPreview) ? '<a href="#" class="wpsseo-button ai-optimize" data-id="wpsseo_description" data-type="description" data-source="' . esc_attr($generateDescriptionFrom) . '" data-tooltip="' . __('(Only Pro) Generate an SEO optimized description.', WPSSEO_TEXT_DOMAIN) . '">&#x1FA84; ' . esc_attr_x('AI', 'Generate button', WPSSEO_TEXT_DOMAIN) . '</a>' : '') . '
						</p>
						<p class="description">' . sprintf(__('Text entered in the Meta Description field is used as the short page description under the title on search engine results pages. Recommended length is %s-%s characters.', WPSSEO_TEXT_DOMAIN), WPSSEO_REC_DESC_MIN_LENGTH, WPSSEO_REC_DESC_MAX_LENGTH) . '</p>
					</td>
				</tr>

				<tr>
					<th scope="row" class="wpsseo-checkbox-label">' . __('Search Engines', WPSSEO_TEXT_DOMAIN) . '</th>
					<td>
						<p>
							<label for="wpsseo_noindex" class="wpsseo-check-default">
								<input type="checkbox" name="wpsseo_noindex" id="wpsseo_noindex" value="1"' . ($noindex ? ' checked' : '') . '>
								<span class="check"></span>
								' . __('Hide this page from search results', WPSSEO_TEXT_DOMAIN) . '
							</label>

							</p><p class="extra-margin">

							<label for="wpsseo_nofollow" class="wpsseo-check-default">
								<input type="checkbox" name="wpsseo_nofollow" id="wpsseo_nofollow" value="1"' . ($nofollow ? ' checked' : '') . '>
								<span class="check"></span>
								' . __('Do not follow links on this page', WPSSEO_TEXT_DOMAIN) . '
							</label>

							' . $visibilityWarning . '
						</p>
					</td>
				</tr>
			</tbody>
		</table>';

		$html .= '<div class="wpsseo-advanced-metabox-wrap">'; // Start advanced metabox
		$html .= '<h3 class="wpsseo-section-headline">' . __('Advanced SEO Settings', WPSSEO_TEXT_DOMAIN) . '</h3>';
		$html .= '<table class="form-table seo-table">
			<tbody>
				<tr>
					<th scope="row" class="wpsseo-checkbox-label">' . __('Meta Tags Cache', WPSSEO_TEXT_DOMAIN) . '</th>
					<td>
						<p><label for="wpsseo_meta_tags_cache" class="wpsseo-check-default">
							<input type="checkbox" name="wpsseo_meta_tags_cache" id="wpsseo_meta_tags_cache" value="1"' . ($metaTagsCache ? ' checked' : '') . '>
							<span class="check"></span>
							' . __('Enable meta tags cache', WPSSEO_TEXT_DOMAIN) . '
						</label></p>
					</td>
				</tr>

				<tr>
					<th scope="row"><label for="wpsseo_canonical">' . __('Canonical URL', WPSSEO_TEXT_DOMAIN) . '<span class="screen-reader-text"> &lt;link rel="canonical"&gt;.</span></label></th>
					<td>
						<p><input class="widefat" type="text" name="wpsseo_canonical" id="wpsseo_canonical" value="' . esc_attr($canonical) . '" placeholder="' . esc_attr($permalink) . '"></p>
						<p class="description">' . __('If left empty, the canonical URL will default to the page URL.', WPSSEO_TEXT_DOMAIN) . '</p>
					</td>
				</tr>

				<tr' . (!$showBreadcrumbs ? ' style="display: none !important;"' : '') . '>
					<th scope="row"><label for="wpsseo_breadcrumb_title"><strong>' . __('Breadcrumb Title', WPSSEO_TEXT_DOMAIN) . '</strong></label></th>
					<td>
						<p class="wpsseo-input-wrapper">
							<input class="widefat" type="text" name="wpsseo_breadcrumb_title" id="wpsseo_breadcrumb_title" value="' . esc_attr($breadcrumbTitle) . '">
							' . (!empty($enableProPreview) ? '<a href="#" class="wpsseo-button ai-optimize" data-id="wpsseo_breadcrumb_title" data-type="breadcrumb" data-source="title" data-tooltip="' . __('(Only Pro) Generate a shortened breadcrumb title based on the content.', WPSSEO_TEXT_DOMAIN) . '">&#x1FA84; ' . esc_attr_x('AI', 'Generate button', WPSSEO_TEXT_DOMAIN) . '</a>' : '') . '
						</p>
						<p class="description">' . __('If left empty, the breadcrumb title will default to the page title.', WPSSEO_TEXT_DOMAIN) . '</p>
					</td>
				</tr>

				<tr>
					<th scope="row"><label for="wpsseo_primary_keyword"><strong>' . __('Primary Keyword', WPSSEO_TEXT_DOMAIN) . '</strong></label></th>
					<td>
						<p class="wpsseo-input-wrapper">
							<input class="widefat" type="text" name="wpsseo_primary_keyword" id="wpsseo_primary_keyword" value="' . esc_attr($primaryKeyword) . '">
							' . (!empty($enableProPreview) ? '<a href="#" class="wpsseo-button ai-optimize" data-id="wpsseo_primary_keyword" data-type="keyword" data-source="content" data-tooltip="' . __('(Only Pro) Generate primary keyword based on the content.', WPSSEO_TEXT_DOMAIN) . '">&#x1FA84; ' . esc_attr_x('AI', 'Generate button', WPSSEO_TEXT_DOMAIN) . '</a>' : '') . '
						</p>
					</td>
				</tr>

				<tr>
					<th scope="row"><label for="wpsseo_secondary_keywords"><strong>' . __('Secondary Keywords', WPSSEO_TEXT_DOMAIN) . '</strong></label></th>
					<td>
						<p><input class="widefat" type="text" name="wpsseo_secondary_keywords" id="wpsseo_secondary_keywords" value="' . esc_attr($secondaryKeywords) . '"></p>
						<p class="description">' . __('For multiple secondary keywords, separate them by commas.', WPSSEO_TEXT_DOMAIN) . '</p>
					</td>
				</tr>

				<tr>
					<th scope="row"><label for="wpsseo_social_image"><strong>' . __('Social Image', WPSSEO_TEXT_DOMAIN) . '</strong></label></th>
					<td>
						<p class="wpsseo-input-wrapper">
							<input type="hidden" name="wpsseo_social_image" id="wpsseo_social_image" value="' . esc_attr($socialImageId) . '">
							<div id="wpsseo_social_image-preview"><div class="image">' . $socialImageObject . '</div></div>
							<input class="wpsseo-upload-image wpsseo-button" type="button" data-id="wpsseo_social_image" value="' . esc_attr__('Select image', WPSSEO_TEXT_DOMAIN) . '">
							<input class="remove-image wpsseo-button" type="button" data-id="wpsseo_social_image" value="' . esc_attr__('Remove image', WPSSEO_TEXT_DOMAIN) . '"' . (empty($socialImageId) ? ' style="display: none;"' : '') . '>
						</p>
					</td>
				</tr>
			</tbody>
		</table>';
		$html .= '</div>'; // End advanced metabox
		$html .= '</div>'; // End metabox wrap
		$html .= '</div>'; // End tab content

		$html .= '<div id="page-scripts" class="tab-content hidden">'; // Start scripts tab
		$html .= '<table class="form-table seo-table">
			<tbody>
				<tr>
					<th scope="row"><label for="wpsseo_header_scripts">' . __('Header Scripts', WPSSEO_TEXT_DOMAIN) . '</label></th>
					<td>
						<p><textarea class="widefat" name="wpsseo_header_scripts" id="wpsseo_header_scripts" rows="8">' . esc_attr($headerScripts) . '</textarea></p>
						<p class="description">' . __('The header script is added to the head section of the website.', WPSSEO_TEXT_DOMAIN) . '</p>
					</td>
				</tr>

				<tr>
					<th scope="row"><label for="wpsseo_body_scripts">' . __('Body scripts', WPSSEO_TEXT_DOMAIN) . '</label></th>
					<td>
						<p><textarea class="widefat" name="wpsseo_body_scripts" id="wpsseo_body_scripts" rows="8">' . esc_attr($bodyScripts) . '</textarea></p>
						<p class="description">' . __('The footer script is added to the bottom section of the website body.', WPSSEO_TEXT_DOMAIN) . '</p>
					</td>
				</tr>
			</tbody>
		</table>';

		$html .= '</div>'; // End scripts tab

		if ($enableProPreview) {
			$html .= '<div id="seo-analysis" class="tab-content hidden">
				<div class="pro-preview preview-seo-health-post">
					' . sprintf(__('%sSEO Health Analysis is a Pro feature.%s%sGet Swift SEO Pro%s', WPSSEO_TEXT_DOMAIN), '<p>', '</p>', '<a href="' . WPSSEO_PLUGIN_URI . '" target="_blank" class="wpsseo-button hero">', '</a>') . '
				</div>
			</div>';
		}

		return $html;
	}

	public function renderSeoMetabox($post)
	{
		echo self::seoMetabox($post);
	}

	public function saveMetabox($postId, $post)
	{
		if (!isset($_POST['save_seo_meta']) || !wp_verify_nonce(wp_unslash($_POST['save_seo_meta']), 'save_seo_meta')) {
			return;
		}

		if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
			return;
		}

		if (!current_user_can('edit_post', $postId)) {
			return;
		}

		$autoTitle = sanitize_text_field(SwiftContent::generateDocumentTitle($post, 'post'));
		$autoDescription = sanitize_text_field(SwiftContent::generateMetaDescription($post, 'post'));
		$metaTitle = sanitize_text_field($_POST['wpsseo_title']);
		$metaDescription = sanitize_textarea_field($_POST['wpsseo_description']);
		$canonical = sanitize_text_field($_POST['wpsseo_canonical']);
		$primaryKeyword = sanitize_text_field($_POST['wpsseo_primary_keyword']);
		$secondaryKeywords = sanitize_text_field($_POST['wpsseo_secondary_keywords']);
		$socialImageId = intval($_POST['wpsseo_social_image'] ?? 0);
		$noindex = isset($_POST['wpsseo_noindex']) && !empty($_POST['wpsseo_noindex']) ? true : false;
		$nofollow = isset($_POST['wpsseo_nofollow']) && !empty($_POST['wpsseo_nofollow']) ? true : false;
		$metaTagsCache = isset($_POST['wpsseo_meta_tags_cache']) && !empty($_POST['wpsseo_meta_tags_cache']) ? 1 : 0;
		$breadcrumbTitle = sanitize_text_field($_POST['wpsseo_breadcrumb_title']);
		$headerScripts = sanitize_text_field($_POST['wpsseo_header_scripts']);
		$bodyScripts = sanitize_text_field($_POST['wpsseo_body_scripts']);

		update_post_meta($postId, '_auto_title', $autoTitle);
		update_post_meta($postId, '_auto_description', $autoDescription);
		update_post_meta($postId, '_genesis_title', $metaTitle);
		update_post_meta($postId, '_genesis_description', $metaDescription);
		update_post_meta($postId, '_genesis_canonical_uri', $canonical);
		update_post_meta($postId, '_genesis_noindex', $noindex);
		update_post_meta($postId, '_genesis_nofollow', $nofollow);
		update_post_meta($postId, '_meta_tags_cache', $metaTagsCache);
		update_post_meta($postId, '_primary_keyword', $primaryKeyword);
		update_post_meta($postId, '_secondary_keywords', $secondaryKeywords);
		update_post_meta($postId, '_social_image_id', $socialImageId);
		update_post_meta($postId, '_breadcrumb_title', $breadcrumbTitle);
		update_post_meta($postId, '_genesis_scripts', $headerScripts);
		update_post_meta($postId, '_genesis_scripts_body', $bodyScripts);

		if (isset($_POST['primary_category'])) {
			$primaryCategoryId = intval($_POST['primary_category']);
			update_post_meta($postId, '_primary_category', $primaryCategoryId);
		}

		delete_transient("wpsseo_meta_tags_post_{$postId}");
		delete_transient("wpsseo_meta_tags_author_{$post->post_author}");
		delete_transient("wpsseo_meta_tags_home_dynamic");

		global $wpdb;
		$wpdb->query($wpdb->prepare("DELETE FROM `{$wpdb->prefix}options` WHERE `option_name` LIKE %s", "_transient%_wpsseo_sitemap_{$post->post_type}%"));

		$taxonomies = get_object_taxonomies($post->post_type);
		if (!empty($taxonomies)) {
			foreach ($taxonomies as $taxonomy) {
				$terms = get_the_terms($post->ID, $taxonomy);

				if (!empty($terms)) {
					foreach ($terms as $term) {
						delete_transient("wpsseo_meta_tags_term_{$term->term_id}");
					}
				}
			}
		}
	}
}
