<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftTaxonomies
{
	public function __construct()
	{
		add_action('admin_init', [$this, 'addFormFields']);
	}

	public static function init()
	{
		return new self();
	}

	public function addFormFields()
	{
		$allTaxonomies = self::getTaxonomies();

		foreach ($allTaxonomies as $taxonomy => $taxonomyData) {
			add_action("{$taxonomy}_edit_form_fields", [$this, 'renderMetabox'], 85);
			add_action("edited_{$taxonomy}", [$this, 'saveMetabox']);
		}
	}

	public static function getTaxonomies()
	{
		$taxonomies = get_taxonomies(['public' => true], 'objects');
		unset($taxonomies['post_format']);
		unset($taxonomies['fb_product_set']);
		unset($taxonomies['product_shipping_class']);

		$taxonomiesReturn = [];
		foreach ($taxonomies as $taxonomy) {
			$label = $taxonomy->label;

			$postType = $taxonomy->object_type[0] ?? null;
			if ($postType) {
				$postTypeObject = get_post_type_object($postType);
				$label .= " ({$postTypeObject->labels->name})";
			}

			$taxonomiesReturn[$taxonomy->name] = $label;
		}

		return $taxonomiesReturn;
	}

	public static function termHasPosts($term, $taxonomy = null)
	{
		global $wpdb;

		$termId = null;
		if ($term && is_numeric($term) && $taxonomy) {
			$termId = $term;
		} elseif ($term && !is_wp_error($term)) {
			$termId = (int) $term->term_id;
			$taxonomy = esc_sql($taxonomy ?? $term->taxonomy);
		}

		if ($termId) {
			$sql = "
				SELECT 1 
				FROM {$wpdb->term_relationships} tr
				INNER JOIN {$wpdb->posts} p ON p.ID = tr.object_id
				INNER JOIN {$wpdb->term_taxonomy} tt ON tt.term_taxonomy_id = tr.term_taxonomy_id
				WHERE tt.term_id = {$termId} 
				AND tt.taxonomy = '{$taxonomy}' 
				AND p.post_status = 'publish'
				LIMIT 1
			";

			return (bool) $wpdb->get_var($sql);
		}

		return false;
	}

	public static function getPrimaryCategory($postId)
	{
		$primaryCategory = null;
		$primaryCategoryId = get_post_meta($postId, '_primary_category', 'true');

		if (empty($primaryCategoryId)) {
			$categories = get_the_category($postId);

			if (!empty($categories)) {
				$primaryCategory = reset($categories);
			}
		} else {
			$primaryCategory = get_term($primaryCategoryId);
		}

		return $primaryCategory;
	}

	public static function getTermIds($taxonomy)
	{
		global $wpdb;

		$termIds = $wpdb->get_col(
			$wpdb->prepare(
				"SELECT `t`.`term_id`
				FROM `{$wpdb->terms}` AS `t`
				INNER JOIN `{$wpdb->term_taxonomy}` AS `tt` ON `t`.`term_id` = `tt`.`term_id`
				WHERE `tt`.`taxonomy` = %s
				AND `t`.`term_id` NOT IN (
					SELECT `term_id` FROM `{$wpdb->termmeta}`
					WHERE `meta_key` = 'noindex' AND `meta_value` = 1
				)",
				$taxonomy
			)
		);

		return $termIds;
	}

	public function renderMetabox($term)
	{
		echo '</table><div class="postbox wpsseo wpsseo-settings"><div class="inside"><h2>' . __('Search Engine Optimization - Swift SEO', WPSSEO_TEXT_DOMAIN) . '</h2>';
		echo SwiftMetaBoxes::seoMetabox($term, 'term');
		echo '</div></div>';
	}

	public function saveMetabox(int $termId)
	{
		if (!isset($_POST['save_seo_meta']) || !wp_verify_nonce(wp_unslash($_POST['save_seo_meta']), 'save_seo_meta')) {
			return;
		}

		$term = get_term($termId);

		$metaKeysToCheck = ['_auto_title', '_auto_description', 'doctitle', 'description', 'noindex', 'nofollow', '_primary_keyword', '_secondary_keywords', '_social_image_url', '_social_image_id', '_breadcrumb_title', '_genesis_scripts', '_genesis_scripts_body', '_wpsseo_analysis_disabled'];
		$existingMetaValues = array_intersect_key(get_term_meta($termId), array_flip($metaKeysToCheck));

		$metaValues = [
			'_auto_title' => sanitize_text_field(SwiftContent::generateDocumentTitle($term, 'term')),
			'_auto_description' => sanitize_text_field(SwiftContent::generateMetaDescription($term, 'term')),
			'doctitle' => sanitize_text_field($_POST['wpsseo_title'] ?? ''),
			'description' => sanitize_textarea_field($_POST['wpsseo_description'] ?? ''),
			'_genesis_canonical_uri' => sanitize_text_field($_POST['wpsseo_canonical'] ?? ''),
			'noindex' => isset($_POST['wpsseo_noindex']) ? '1' : '0',
			'nofollow' => isset($_POST['wpsseo_nofollow']) ? '1' : '0',
			'_meta_tags_cache' => isset($_POST['wpsseo_meta_tags_cache']) ? '1' : '0',
			'_primary_keyword' => sanitize_text_field($_POST['wpsseo_primary_keyword'] ?? ''),
			'_secondary_keywords' => sanitize_text_field($_POST['wpsseo_secondary_keywords'] ?? ''),
			'_social_image_id' => $_POST['wpsseo_social_image'] ?? '0',
			'_breadcrumb_title' => sanitize_text_field($_POST['wpsseo_breadcrumb_title'] ?? ''),
			'_genesis_scripts' => sanitize_text_field($_POST['wpsseo_header_scripts'] ?? ''),
			'_genesis_scripts_body' => sanitize_text_field($_POST['wpsseo_body_scripts'] ?? ''),
		];

		foreach ($metaValues as $key => $value) {
			if (($existingMetaValues[$key][0] ?? '') !== $value) {
				update_term_meta($termId, $key, $value);
			}
		}

		delete_transient("wpsseo_meta_tags_term_{$termId}");

		global $wpdb;
		$wpdb->query($wpdb->prepare("DELETE FROM `{$wpdb->prefix}options` WHERE `option_name` LIKE %s", "_transient%_wpsseo_sitemap_{$term->taxonomy}%"));

		$postIds = get_posts([
			'numberposts' => -1,
			'fields' => 'ids',
			'tax_query' => [
				[
					'taxonomy' => $term->taxonomy,
					'field' => 'term_id',
					'terms' => $termId
				]
			]
		]);

		if (!empty($postIds)) {
			foreach ($postIds as $postId) {
				delete_transient("wpsseo_meta_tags_post_{$postId}");
			}
		}
	}
}
