<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftCategories
{
	public function __construct()
	{
		$removeCategoryBase = (bool) get_option('wpsseo_remove_category_base', false);
		if ($removeCategoryBase) {
			add_action('created_category', 'flush_rewrite_rules');
			add_action('delete_category', 'flush_rewrite_rules');
			add_action('edited_category', 'flush_rewrite_rules');
			add_action('init', [$this, 'permastruct']);
			add_filter('category_rewrite_rules', [$this, 'rewriteRules']);
			add_filter('query_vars', [$this, 'queryVars']); // Adds 'category_redirect' query variable
			add_filter('request', [$this, 'request']); // Redirects if 'category_redirect' is set
		}

		add_filter('post_link_category', [$this, 'usePrimaryCategoryForPermalinks'], 10, 3);
		add_action('template_redirect', [$this, 'redirectToPrimaryCategory']);
	}

	public static function init()
	{
		return new self();
	}

	public function permastruct()
	{
		global $wp_rewrite;

		$wp_rewrite->extra_permastructs['category']['struct'] = '%category%';
	}

	public function rewriteRules($categoryRewrite)
	{
		global $wp_rewrite;

		$categoryRewrite = [];

		/* WPML is present: temporary disable terms_clauses filter to get all categories for rewrite */
		if (class_exists('Sitepress')) {
			global $sitepress;

			remove_filter('terms_clauses', [$sitepress, 'terms_clauses']);
			$categories = get_categories(['hide_empty' => false]);
			add_filter('terms_clauses', [$sitepress, 'terms_clauses'], 10, 4);
		} else {
			$categories = get_categories(['hide_empty' => false]);
		}

		foreach ($categories as $category) {
			$categoryNicename = $category->slug;

			if ($category->parent == $category->cat_ID) {
				$category->parent = 0;
			} elseif ($category->parent != 0) {
				$categoryNicename = get_category_parents($category->parent, false, '/', true) . $categoryNicename;
			}

			$categoryRewrite['(' . $categoryNicename . ')/(?:feed/)?(feed|rdf|rss|rss2|atom)/?$'] = 'index.php?category_name=$matches[1]&feed=$matches[2]';
			$categoryRewrite["({$categoryNicename})/{$wp_rewrite->pagination_base}/?([0-9]{1,})/?$"] = 'index.php?category_name=$matches[1]&paged=$matches[2]';
			$categoryRewrite['(' . $categoryNicename . ')/?$'] = 'index.php?category_name=$matches[1]';
		}

		// Redirect support from Old Category Base
		$oldCategoryBase = get_option('category_base', 'category');
		$oldCategoryBase = trim($oldCategoryBase, '/');
		$categoryRewrite[$oldCategoryBase . '/(.*)$'] = 'index.php?category_redirect=$matches[1]';

		return $categoryRewrite;
	}

	public function queryVars($publicQueryVars)
	{
		$publicQueryVars[] = 'categoryRedirect';

		return $publicQueryVars;
	}

	public function request($queryVars)
	{
		if (isset($queryVars['categoryRedirect'])) {
			$catLink = trailingslashit(get_option('home')) . user_trailingslashit($queryVars['categoryRedirect'], 'category');
			status_header(301);
			header("Location: $catLink");
			exit;
		}

		return $queryVars;
	}

	public function usePrimaryCategoryForPermalinks($cat, $cats, $post)
	{
		$primaryCategory = get_post_meta($post->ID, '_primary_category', true);

		if ($primaryCategory && term_exists((int) $primaryCategory)) {
			$cat = get_term($primaryCategory);
		}

		return $cat;
	}

	public function redirectToPrimaryCategory()
	{
		$post = get_post();

		// If the post is single.
		if (!empty($post->ID) && is_single($post->ID)) {
			if (SwiftUtilities::isFeed()) {
				return;
			}

			if ($this->isAmp()) {
				return;
			}

			// If the post has multiple categories assigned.
			$cats = wp_get_post_categories($post->ID);

			if (count($cats) > 1) {
				global $wp;

				$permalink = get_permalink($post->ID);

				// If the primary category isn't in the URL, redirect.
				if (!empty($permalink) && !empty($wp->request)) {
					if (strpos($permalink, $wp->request) === false && strpos($wp->request, $permalink) === false) {
						wp_safe_redirect($permalink, 301, 'Swift SEO - Primary Category');
						exit;
					}
				}
			}
		}
	}

	private function isAmp()
	{
		global $wp;

		if (function_exists('kadence_is_amp') && kadence_is_amp()) {
			return true;
		}

		if (function_exists('is_amp_endpoint') && is_amp_endpoint()) {
			return true;
		}

		if (isset($_GET['amp']) && $_GET['amp'] !== '0' && $_GET['amp'] !== 'false') {
			return true;
		}

		if (function_exists('amp_is_request') && amp_is_request()) {
			return true;
		}

		if (class_exists('AMP_Theme_Support') && method_exists('AMP_Theme_Support', 'is_amp_endpoint') && AMP_Theme_Support::is_amp_endpoint()) {
			return true;
		}

		if (substr($wp->request, -5) === '/amp/') {
			return true;
		}

		return false;
	}
}
