<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftMetaTags
{
	public static $postTypes;
	public static $taxonomies;
	public $objectId = null;
	public $objectType = null;

	public function __construct()
	{
		add_action('template_redirect', [$this, 'setPostTypesAndTaxonomies'], 1);

		add_action('wp_head', [$this, 'addHeaderMeta'], 1);
		add_filter('wp_robots', [$this, 'modifyRobotsMeta']);
		add_filter('pre_get_document_title', [$this, 'modifyDocumentTitle']);

		add_action('wp_head', [$this, 'addAnalytics']);
		add_action('wp_footer', [$this, 'addBodyScripts']);
	}

	public static function init()
	{
		return new self();
	}

	public function setPostTypesAndTaxonomies()
	{
		self::$postTypes = array_keys(SwiftPostTypes::getPostTypes());
		self::$taxonomies = array_keys(SwiftTaxonomies::getTaxonomies());

		if (is_author()) {
			$this->objectId = get_the_author_meta('ID');
			$this->objectType = 'author';
		} elseif (is_archive()) {
			if (WPSSEO_IS_WOOCOMMERCE && is_shop()) {
				$shopPageId = (int) get_option('woocommerce_shop_page_id', 0);

				if (!empty($shopPageId)) {
					$this->objectId = $shopPageId;
					$this->objectType = 'post';
				}
			} else {
				$term = get_queried_object();

				if (!empty($term->term_id)) {
					$this->objectId = $term->term_id;
					$this->objectType = 'term';
				}
			}
		} elseif (self::isDynamicFrontPage()) {
			$this->objectId = 'dynamic';
			$this->objectType = 'home';
		} elseif (self::isFrontPage()) {
			$pageId = get_option('page_on_front');

			$this->objectId = $pageId;
			$this->objectType = 'post';
		} elseif (self::isHome()) {
			$pageId = get_option('page_for_posts');

			$this->objectId = $pageId;
			$this->objectType = 'post';
		} elseif (is_singular(self::$postTypes)) {
			$post = get_post();

			$this->objectId = $post->ID;
			$this->objectType = 'post';
		}
	}

	private static function isSupportedTaxonomy($term)
	{
		if (is_a($term, 'WP_Term') && !empty($term->taxonomy)) {
			return (bool) in_array($term->taxonomy, self::$taxonomies);
		}

		return false;
	}

	public static function isFrontPage()
	{
		if (is_front_page() && !is_home()) {
			return true;
		}

		return false;
	}

	public static function isHome()
	{
		if (is_home() && !is_front_page()) {
			return true;
		}

		return false;
	}

	public static function isDynamicFrontPage()
	{
		if (is_home() && is_front_page()) {
			return true;
		}

		return false;
	}

	private function getDocumentTitle($doctitle = '')
	{
		$companyName = get_option('wpsseo_company_name', get_option('blogname'));
		$appendToTitleTag = (bool) get_option('wpsseo_append_to_title_tag', true);
		$append = '';

		if ($appendToTitleTag) {
			$append = ' &bull; ' . $companyName;
		}

		if (is_404()) {
			$doctitle = __('404 - Page not found', WPSSEO_TEXT_DOMAIN);
		} elseif (is_search()) {
			$query = get_search_query();
			$doctitle = !empty($query) ? sprintf(__('Search for %s', WPSSEO_TEXT_DOMAIN), esc_html($query)) : __('Empty search', WPSSEO_TEXT_DOMAIN);
		} elseif (is_author()) {
			$authorId = get_the_author_meta('ID');

			$title = !empty($authorId) ? get_user_meta($authorId, 'doctitle', true) : null;
			$title = !empty($authorId) && empty($title) ? get_user_meta($authorId, 'headline', true) : $title;
			$doctitle = !empty($title) ? $title : $doctitle;
			$doctitle = !empty($doctitle) ? $doctitle : get_user_meta($authorId, '_auto_title', true);
			$doctitle = !empty($doctitle) ? $doctitle : __('Archives for ', WPSSEO_TEXT_DOMAIN) . get_the_author_meta('display_name');
		} elseif (is_date()) {
			if (is_day()) {
				$doctitle = __('Archives for ', WPSSEO_TEXT_DOMAIN) . get_the_date();
			} elseif (is_month()) {
				$doctitle = __('Archives for ', WPSSEO_TEXT_DOMAIN) . single_month_title(' ', false);
			} elseif (is_year()) {
				$doctitle = __('Archives for ', WPSSEO_TEXT_DOMAIN) . get_query_var('year');
			}
		} elseif (is_archive()) {
			if (WPSSEO_IS_WOOCOMMERCE && is_shop()) {
				$shopPageId = (int) get_option('woocommerce_shop_page_id', 0);

				if (!empty($shopPageId)) {
					$title = get_post_meta($shopPageId, '_genesis_title', true);
					$doctitle = !empty($title) ? $title : $doctitle;
					$doctitle = !empty($doctitle) ? $doctitle : get_post_meta($shopPageId, '_auto_title', true);
					$doctitle = !empty($doctitle) ? $doctitle : get_the_title($shopPageId);
				}
			} elseif (is_category() || is_tag() || is_tax()) {
				$term = get_queried_object();

				$title = !empty($term) ? get_term_meta($term->term_id, 'doctitle', true) : null;
				$title = !empty($term->term_id) && empty($title) ? get_term_meta($term->term_id, 'headline', true) : $title;
				$doctitle = !empty($title) ? $title : $doctitle;
				$doctitle = !empty($doctitle) ? $doctitle : get_term_meta($term->term_id, '_auto_title', true);
				$doctitle = !empty($doctitle) ? $doctitle : $term->name;
			}
		} elseif (self::isDynamicFrontPage()) {
			$title = (string) get_option('wpsseo_home_seo_title');
			$doctitle = !empty($title) ? $title : $doctitle;
			$doctitle = !empty($doctitle) ? $doctitle : $companyName;
		} elseif (self::isFrontPage()) {
			$pageId = get_option('page_on_front');

			$title = get_post_meta($pageId, '_genesis_title', true);
			$doctitle = !empty($title) ? $title : $doctitle;
			$doctitle = !empty($doctitle) ? $doctitle : get_post_meta($pageId, '_auto_title', true);
			$doctitle = !empty($doctitle) ? $doctitle : get_the_title($pageId);
		} elseif (self::isHome()) {
			$pageId = get_option('page_for_posts');

			$title = get_post_meta($pageId, '_genesis_title', true);
			$doctitle = !empty($title) ? $title : $doctitle;
			$doctitle = !empty($doctitle) ? $doctitle : get_post_meta($pageId, '_auto_title', true);
			$doctitle = !empty($doctitle) ? $doctitle : get_the_title($pageId);
		} elseif (is_singular(self::$postTypes)) {
			$post = get_post();

			$title = !empty($post) ? get_post_meta($post->ID, '_genesis_title', true) : null;
			$doctitle = !empty($title) ? $title : $doctitle;
			$doctitle = !empty($doctitle) ? $doctitle : get_post_meta($post->ID, '_auto_title', true);
			$doctitle = !empty($doctitle) ? $doctitle : get_the_title($post->ID);
		}

		if (SwiftDomain::isPaginated() && !SwiftDomain::isFirstPage()) {
			$doctitle .= ' &bull; ' . sprintf(__('Page %s of %s', ''), get_query_var('paged'), SwiftDomain::maxPages());
		}

		$totalLength = mb_strlen((string) html_entity_decode($doctitle . $append));
		if (!empty($doctitle) && $totalLength <= WPSSEO_REC_TITLE_MAX_LENGTH && strstr($doctitle, $companyName) === false) {
			$doctitle .= $append;
		}

		return apply_filters('wpsseo_meta_doctitle', trim($doctitle));
	}

	private function getMetaDescription()
	{
		$metaDescription = '';

		if (is_author()) {
			$userId = get_the_author_meta('ID');

			$metaDescription = get_user_meta($userId, 'meta_description', true);
			$metaDescription = !empty($metaDescription) ? $metaDescription : get_user_meta($userId, '_auto_description', true);
			$metaDescription = !empty($metaDescription) ? $metaDescription : SwiftContent::generateMetaDescription($userId, 'author');
		} elseif (self::isDynamicFrontPage()) {
			$metaDescription = get_option('wpsseo_home_seo_description');
		} elseif (self::isFrontPage()) {
			$pageId = get_option('page_on_front');

			if (!empty($pageId)) {
				$metaDescription = get_post_meta($pageId, '_genesis_description', true);
				$metaDescription = !empty($metaDescription) ? $metaDescription : get_post_meta($pageId, '_auto_description', true);
				$metaDescription = !empty($metaDescription) ? $metaDescription : SwiftContent::generateMetaDescription($pageId, 'post');
			}
		} elseif (self::isHome()) {
			$pageId = get_option('page_for_posts');

			if (!empty($pageId)) {
				$metaDescription = get_post_meta($pageId, '_genesis_description', true);
				$metaDescription = !empty($metaDescription) ? $metaDescription : get_post_meta($pageId, '_auto_description', true);
				$metaDescription = !empty($metaDescription) ? $metaDescription : SwiftContent::generateMetaDescription($pageId, 'post');
			}
		} elseif (is_singular(self::$postTypes)) {
			$post = get_post();

			$metaDescription = get_post_meta($post->ID, '_genesis_description', true);
			$metaDescription = !empty($metaDescription) ? $metaDescription : get_post_meta($post->ID, '_auto_description', true);
			$metaDescription = !empty($metaDescription) ? $metaDescription : SwiftContent::generateMetaDescription($post, 'post');
		} elseif (is_archive()) {
			if (WPSSEO_IS_WOOCOMMERCE && is_shop()) {
				$shopPageId = (int) get_option('woocommerce_shop_page_id', 0);

				if (!empty($shopPageId)) {
					$shopPage = get_post($shopPageId);
					$metaDescription = get_post_meta($shopPage->ID, '_genesis_description', true);
					$metaDescription = !empty($metaDescription) ? $metaDescription : get_post_meta($shopPage->ID, '_auto_description', true);
					$metaDescription = !empty($metaDescription) ? $metaDescription : SwiftContent::generateMetaDescription($shopPage, 'post');
				}
			} elseif ($term = get_queried_object()) {
				if (self::isSupportedTaxonomy($term)) {
					$metaDescription = get_term_meta($term->term_id, 'description', true);
					$metaDescription = !empty($metaDescription) ? $metaDescription : get_term_meta($term->term_id, '_auto_description', true);
					$metaDescription = !empty($metaDescription) ? $metaDescription : SwiftContent::generateMetaDescription($term, 'term');
				}
			}
		}

		if (SwiftDomain::isPaginated() && !SwiftDomain::isFirstPage()) {
			if (!empty($metaDescription)) {
				$metaDescription .= ' &bull; ' . sprintf(__('Page %s of %s', ''), get_query_var('paged'), SwiftDomain::maxPages());
			}
		}

		return apply_filters('wpsseo_meta_description', trim($metaDescription));
	}

	private function getCanonicalUrl()
	{
		global $wp_query;

		$canonicalUrl = '';
		$isPaged = is_paged();

		if (is_search()) {
			$canonicalUrl = get_search_link();
		} elseif (is_author()) {
			$userId = (int) $wp_query->get_queried_object_id();

			if (!empty($userId)) {
				$customCanonical = get_user_meta($userId, '_genesis_canonical_uri', true);

				if ($customCanonical) {
					$canonicalUrl = $customCanonical;
				} elseif ($isPaged) {
					$canonicalUrl = WPSSEO_CURRENT_URL;
				} else {
					$canonicalUrl = get_author_posts_url($userId);
				}
			}
		} elseif (is_date()) {
			if (is_day()) {
				$canonicalUrl = get_day_link(get_query_var('year'), get_query_var('monthnum'), get_query_var('day'));
			} elseif (is_month()) {
				$canonicalUrl = get_month_link(get_query_var('year'), get_query_var('monthnum'));
			} elseif (is_year()) {
				$canonicalUrl = get_year_link(get_query_var('year'));
			}
		} elseif (is_post_type_archive()) {
			if ($isPaged) {
				$canonicalUrl = WPSSEO_CURRENT_URL;
			} else {
				$canonicalUrl = get_post_type_archive_link(get_query_var('post_type'));
			}
		} elseif (is_archive()) {
			if (WPSSEO_IS_WOOCOMMERCE && is_shop()) {
				$shopPageId = (int) get_option('woocommerce_shop_page_id', 0);

				if (!empty($shopPageId)) {
					$customCanonical = get_post_meta($shopPageId, '_genesis_canonical_uri', true);

					if ($customCanonical) {
						$canonicalUrl = $customCanonical;
					} elseif ($isPaged) {
						$canonicalUrl = WPSSEO_CURRENT_URL;
					} else {
						$canonicalUrl = get_permalink($shopPageId);
					}
				}
			} elseif (is_category() || is_tag() || is_tax()) {
				$termId = (int) $wp_query->get_queried_object_id();

				if (!empty($termId)) {
					$customCanonical = get_term_meta($termId, '_genesis_canonical_uri', true);

					if ($customCanonical) {
						$canonicalUrl = $customCanonical;
					} elseif ($isPaged) {
						$canonicalUrl = WPSSEO_CURRENT_URL;
					} else {
						$taxonomy = $wp_query->queried_object->taxonomy;
						$canonicalUrl = get_term_link($termId, $taxonomy);
					}
				}
			}
		} elseif (self::isDynamicFrontPage() || self::isFrontPage()) {
			$canonicalUrl = home_url();
		} elseif (self::isHome()) {
			$blogHomeId = get_option('page_for_posts');
			$canonicalUrl = !empty($blogHomeId) ? get_permalink($blogHomeId) : home_url();
		} elseif (is_singular(self::$postTypes)) {
			$postId = $wp_query->get_queried_object_id();

			if (!empty($postId)) {
				$customCanonical = (int) get_post_meta($postId, '_genesis_canonical_uri', true);

				if ($customCanonical) {
					$canonicalUrl = $customCanonical;
				} elseif ($isPaged) {
					$canonicalUrl = WPSSEO_CURRENT_URL;
				} else {
					$canonicalUrl = get_permalink($postId);
				}
			}
		}

		return apply_filters('wpsseo_meta_canonical_url', $this->maybeAddTrailingSlash($canonicalUrl ?: WPSSEO_CURRENT_URL));
	}

	private function maybeAddTrailingSlash($url)
	{
		$permalinkStructure = get_option('permalink_structure');
		$trailingSlash = !empty($permalinkStructure) && substr($permalinkStructure, -1) === '/';

		return $trailingSlash && strpos($url, '?') === false ? trailingslashit($url) : $url;
	}

	public static function getSocialImage()
	{
		$socialImageArray = [];
		$postId = null;

		if (is_author()) {
			$userId = get_the_author_meta('ID');

			$socialImageId = get_user_meta($userId, '_social_image_id', true);
		} elseif (self::isFrontPage()) {
			$postId = get_option('page_on_front');

			if (!empty($postId)) {
				$socialImageId = get_post_meta($postId, '_social_image_id', true);
			}
		} elseif (self::isHome()) {
			$postId = get_option('page_for_posts');

			if (!empty($postId)) {
				$socialImageId = get_post_meta($postId, '_social_image_id', true);
			}
		} elseif (is_singular(self::$postTypes)) {
			$post = get_post();
			$postId = $post->ID;

			$socialImageId = get_post_meta($postId, '_social_image_id', true);
		} elseif ($term = get_queried_object()) {
			if (self::isSupportedTaxonomy($term)) {
				$socialImageId = get_term_meta($term->term_id, '_social_image_id', true);
			}
		}

		if (empty($socialImageId) && !empty($postId)) {
			$socialImageId = get_post_thumbnail_id($postId);
		}

		if (empty($socialImageId)) {
			$socialImageId = get_option('wpsseo_default_social_image');
		}

		if (!empty($socialImageId)) {
			$socialImage = wp_get_attachment_image_src($socialImageId, 'social-image');

			if (!empty($socialImage)) {
				$url = $socialImage[0];
				$width = $socialImage[1];
				$height = $socialImage[2];
				$alt = get_post_meta($socialImageId, '_wp_attachment_image_alt', true);

				if (empty($alt)) {
					$alt = mb_convert_case(trim(preg_replace('%\s*[-_\s]+\s*%', ' ', get_the_title($socialImageId))), MB_CASE_TITLE, "UTF-8");
				}

				$socialImageArray = [
					'url' => $url,
					'width' => $width,
					'height' => $height,
					'alt' => $alt,
				];
			}
		}

		return apply_filters('wpsseo_meta_social_image', $socialImageArray);
	}

	public function addHeaderMeta()
	{
		if (is_admin() || is_404()) {
			return;
		}

		remove_action('wp_head', 'rel_canonical');

		$metaTags = '';
		$transientName = '';
		$useTransient = false;

		if (WPSSEO_IS_PUBLIC) {
			if (!empty($this->objectId) && !empty($this->objectType)) {
				if ($this->objectType === 'author') {
					$metaTagsCache = get_user_meta($this->objectId, '_meta_tags_cache', true);
				} elseif ($this->objectType === 'term') {
					$metaTagsCache = get_term_meta($this->objectId, '_meta_tags_cache', true);
				} elseif ($this->objectType === 'post') {
					$metaTagsCache = get_post_meta($this->objectId, '_meta_tags_cache', true);
				}

				if (isset($metaTagsCache)) {
					$useTransient = (bool) ($metaTagsCache === '' ? true : (bool) $metaTagsCache);
				}

				$transientName = "wpsseo_meta_tags_{$this->objectType}_{$this->objectId}";

				if (SwiftDomain::isPaginated() && !SwiftDomain::isFirstPage()) {
					$transientName .= '_page_' . get_query_var('paged');
				}
			}

			if ($useTransient) {
				$metaTags = get_transient($transientName);

				if (!empty($metaTags)) {
					echo $metaTags;

					return;
				}
			}
		}

		$prefetchMeta = '';
		$regularMeta = '';
		$ogMeta = '';
		$twitterMeta = '';
		$analyticsMeta = '';
		$verificationMeta = '';

		$blogName = apply_filters('wpsseo_meta_blog_name', get_option('blogname'));
		$metaTitle = $this->getDocumentTitle();
		$metaDescription = $this->getMetaDescription();
		$canonicalUrl = $this->getCanonicalUrl();
		$socialImage = self::getSocialImage();
		$facebookAppId = get_option('wpsseo_facebook_app_id');
		$twitterSiteId = get_option('wpsseo_twitter_site_id');
		$pinterestVerificationCode = get_option('wpsseo_pinterest_verification');
		$gscVerification = get_option('wpsseo_gsc_verification');
		$bingVerification = get_option('wpsseo_bing_verification');
		$yandexVerification = get_option('wpsseo_yandex_verification');
		$baiduVerification = get_option('wpsseo_baidu_verification');
		$adultRating = get_option('wpsseo_adult_rating', false);

		if (!empty($socialImage)) {
			$ogMeta .= '<meta property="og:image" content="' . esc_url($socialImage['url']) . '">' . "\n";
			$ogMeta .= '<meta property="og:image:width" content="' . esc_attr($socialImage['width']) . '">' . "\n";
			$ogMeta .= '<meta property="og:image:height" content="' . esc_attr($socialImage['height']) . '">' . "\n";
			$ogMeta .= '<meta property="og:image:alt" content="' . esc_attr($socialImage['alt']) . '">' . "\n";
			$twitterMeta .= '<meta name="twitter:card" content="summary_large_image">' . "\n";
			$twitterMeta .= '<meta name="twitter:image" content="' . esc_url($socialImage['url']) . '">' . "\n";
			$twitterMeta .= '<meta name="twitter:image:width" content="' . esc_attr($socialImage['width']) . '">' . "\n";
			$twitterMeta .= '<meta name="twitter:image:height" content="' . esc_attr($socialImage['height']) . '">' . "\n";
			$twitterMeta .= '<meta name="twitter:image:alt" content="' . esc_attr($socialImage['alt']) . '">' . "\n";
		}

		if (!empty($metaTitle)) {
			$ogMeta .= '<meta property="og:title" content="' . esc_attr($metaTitle) . '">' . "\n";
			$twitterMeta .= '<meta name="twitter:title" content="' . esc_attr($metaTitle) . '">' . "\n";
		}

		if (!empty($metaDescription)) {
			$regularMeta .= '<meta name="description" content="' . esc_attr($metaDescription) . '">' . "\n";
			$ogMeta .= '<meta property="og:description" content="' . esc_attr($metaDescription) . '">' . "\n";
			$twitterMeta .= '<meta name="twitter:description" content="' . esc_attr($metaDescription) . '">' . "\n";
		}

		if (!empty($canonicalUrl)) {
			$regularMeta .= '<link rel="canonical" href="' . esc_url($canonicalUrl) . '">' . "\n";
			$ogMeta .= '<meta property="og:url" content="' . esc_url($canonicalUrl) . '">' . "\n";
		}

		$addExtendedMeta = (bool) get_option('wpsseo_add_extended_meta', true);
		if ($addExtendedMeta) {
			if ($this->objectType === 'home' || $this->objectType === 'term') {
				global $wp_query;

				$postCount = (int) $wp_query->post_count ?? 0;

				$ogMeta .= '<meta property="og:type" content="website">' . "\n";
				$twitterMeta .= '<meta name="twitter:label1" content="' . esc_attr(__('Items', WPSSEO_TEXT_DOMAIN)) . '">' . "\n";
				$twitterMeta .= '<meta name="twitter:data1" content="' . esc_attr($postCount) . '">' . "\n";
			} elseif ($this->objectType === 'author') {
				$ogMeta .= '<meta property="og:type" content="profile">' . "\n";
				$ogMeta .= '<meta property="profile:first_name" content="' . esc_attr(get_the_author_meta('first_name', $this->objectId)) . '">' . "\n";
				$ogMeta .= '<meta property="profile:last_name" content="' . esc_attr(get_the_author_meta('last_name', $this->objectId)) . '">' . "\n";
			} elseif ($this->objectType === 'post') {
				$modifiedTime = get_the_modified_time('c', $this->objectId);
				$ogType = is_singular('page') ? 'website' : 'article';
				$ogMeta .= '<meta property="og:type" content="' . $ogType . '">' . "\n";
				$ogMeta .= '<meta property="og:updated_time" content="' . esc_attr($modifiedTime) . '">' . "\n";
			}
		} else {
			$ogMeta .= '<meta property="og:type" content="website">' . "\n";
		}

		if (SwiftDomain::isPaginated()) {
			if (!SwiftDomain::isFirstPage()) {
				$regularMeta .= '<link rel="prev" href="' . esc_url(get_previous_posts_page_link()) . '">' . "\n";
				$prefetchMeta .= '<link rel="prefetch" href="' . esc_url(get_previous_posts_page_link()) . '">' . "\n";
			}

			if (!SwiftDomain::isLastPage()) {
				global $wp_query;

				$regularMeta .= '<link rel="next" href="' . esc_url(get_next_posts_page_link($wp_query->max_num_pages)) . '">' . "\n";
				$prefetchMeta .= '<link rel="prefetch" href="' . esc_url(get_next_posts_page_link($wp_query->max_num_pages)) . '">' . "\n";
			}
		}

		if (!empty($adultRating)) {
			$regularMeta .= '<meta name="rating" content="adult">' . "\n";
		}

		if (!empty($facebookAppId)) {
			$analyticsMeta .= '<meta property="fb:app_id" content="' . esc_attr($facebookAppId) . '">' . "\n";
		}

		if (!empty($twitterSiteId)) {
			$twitterMeta .= '<meta name="twitter:site" content="' . esc_attr($twitterSiteId) . '">' . "\n";
		}

		if (!empty($pinterestVerificationCode)) {
			$analyticsMeta .= '<meta name="p:domain_verify" content="' . esc_attr($pinterestVerificationCode) . '">' . "\n";
		}

		if (!empty($gscVerification)) {
			$verificationMeta .= '<meta name="google-site-verification" content="' . esc_attr($gscVerification) . '">' . "\n";
		}

		if (!empty($bingVerification)) {
			$verificationMeta .= '<meta name="msvalidate.01" content="' . esc_attr($bingVerification) . '">' . "\n";
		}

		if (!empty($yandexVerification)) {
			$verificationMeta .= '<meta name="yandex-verification" content="' . esc_attr($yandexVerification) . '">' . "\n";
		}

		if (!empty($baiduVerification)) {
			$verificationMeta .= '<meta name="baidu-site-verification" content="' . esc_attr($baiduVerification) . '">' . "\n";
		}

		$removeCountryCode = (bool) get_option('wpsseo_remove_country_code', true);
		if (!$removeCountryCode) {
			$ogMeta .= '<meta property="og:locale" content="' . esc_attr(get_locale()) . '">' . "\n";
		}

		$ogMeta .= '<meta property="og:site_name" content="' . esc_attr($blogName) . '">' . "\n";

		$metaTags .= "<!-- Swift SEO -->\n";
		$metaTags .= apply_filters('wpsseo_meta_tags', $regularMeta . $prefetchMeta . $ogMeta . $twitterMeta . $analyticsMeta . $verificationMeta);
		$metaTags .= "<!-- Swift SEO End -->\n";

		if ($useTransient) {
			set_transient($transientName, $metaTags, WEEK_IN_SECONDS);
		}

		echo $metaTags;
	}

	public function modifyDocumentTitle($doctitle)
	{
		if (is_feed()) {
			return;
		}

		$doctitle = $this->getDocumentTitle($doctitle);

		return $doctitle;
	}

	public function modifyRobotsMeta($directives)
	{
		if (is_admin()) {
			return ['noindex' => true, 'nofollow' => true, 'noarchive' => true];
		}

		if (is_404()) {
			return ['noindex' => true, 'noarchive' => true];
		}

		$blogPublic = (bool) get_option('blog_public');

		// If the website is not public, return the default
		if (!$blogPublic) {
			return $directives;
		}

		$additionalDirectives = [];

		if (is_singular(self::$postTypes)) {
			$post = get_post();

			// If post is not found, simply return the default
			if (empty($post)) {
				return $directives;
			}

			$noindex = (bool) get_post_meta($post->ID, '_genesis_noindex', true);
			$nofollow = (bool) get_post_meta($post->ID, '_genesis_nofollow', true);
			$noindexPostTypes = get_option('wpsseo_noindex_post_types');
			$nofollowPostTypes = get_option('wpsseo_nofollow_post_types');

			// Now, we established that we're in a public post, and want to do our own robots meta. Unset whatever parameters you are modifying
			unset($directives['noindex'], $directives['nofollow']);

			// If noindex is set, add it to the directives array
			if (!empty($noindexPostTypes) && in_array($post->post_type, array_keys($noindexPostTypes))) {
				$noindex = true;
			}

			// If nofollow is set, add it to the directives array
			if (!empty($nofollowPostTypes) && in_array($post->post_type, array_keys($nofollowPostTypes))) {
				$nofollow = true;
			}

			if (WPSSEO_IS_WOOCOMMERCE) {
				if (is_cart() || is_checkout() || is_account_page() || is_view_order_page() || is_checkout_pay_page() || is_edit_account_page() || is_order_received_page() || is_add_payment_method_page() || is_lost_password_page()) {
					$noindex = true;
				}
			}

			if (!empty($noindex)) {
				$directives['noindex'] = true;
			}

			if (!empty($nofollow)) {
				$directives['nofollow'] = true;
			}
		}

		if ($term = get_queried_object()) {
			if (self::isSupportedTaxonomy($term)) {
				$noindex = (bool) get_term_meta($term->term_id, 'noindex', true);
				$nofollow = (bool) get_term_meta($term->term_id, 'nofollow', true);
				$noindexTaxonomies = get_option('wpsseo_noindex_taxonomies');
				$nofollowTaxonomies = get_option('wpsseo_nofollow_taxonomies');

				unset($directives['noindex'], $directives['nofollow']);

				// If noindex is set, add it to the directives array
				if (!empty($noindexTaxonomies) && in_array($term->taxonomy, array_keys($noindexTaxonomies))) {
					$noindex = true;
				}

				if (empty($noindex)) {
					$noindexEmptyTerms = (bool) get_option('wpsseo_noindex_empty_terms', true);

					if ($noindexEmptyTerms && SwiftTaxonomies::termHasPosts($term) === false) {
						$noindex = true;
					}
				}

				// If nofollow is set, add it to the directives array
				if (!empty($nofollowTaxonomies) && in_array($term->taxonomy, array_keys($nofollowTaxonomies))) {
					$nofollow = true;
				}

				if (!empty($noindex)) {
					$directives['noindex'] = true;
				}

				if (!empty($nofollow)) {
					$directives['nofollow'] = true;
				}
			}
		}

		// Archives
		if (is_author() || is_date() || is_search()) {
			$noindex = false;
			$nofollow = false;
			$type = '';

			if (is_author()) {
				$userId = get_the_author_meta('ID');

				$noindex = (bool) get_user_meta($userId, 'noindex', true);
				$nofollow = (bool) get_user_meta($userId, 'nofollow', true);
				$type = 'author';
			}

			if (is_date()) {
				$type = 'date';
			}

			if (is_search()) {
				$type = 'search';
			}

			$noindexArchives = get_option('wpsseo_noindex_archives');
			$nofollowArchives = get_option('wpsseo_noindex_archives');

			unset($directives['noindex'], $directives['nofollow']);

			// If noindex is set, add it to the directives array
			if (!empty($noindexArchives) && in_array($type, array_keys($noindexArchives))) {
				$noindex = true;
			}

			// If nofollow is set, add it to the directives array
			if (!empty($nofollowArchives) && in_array($type, array_keys($nofollowArchives))) {
				$nofollow = true;
			}

			if (!empty($noindex)) {
				$directives['noindex'] = true;
			}

			if (!empty($nofollow)) {
				$directives['nofollow'] = true;
			}
		}

		if (self::isDynamicFrontPage()) {
			$noindex = get_option('wpsseo_home_noindex', false);

			unset($directives['noindex'], $directives['nofollow']);

			if (!empty($noindex)) {
				$directives['noindex'] = true;
			}
		}

		unset($directives['max-image-preview']);

		if (empty($directives['noindex'])) {
			$additionalDirectives['index'] = true;
			$directives['max-snippet:-1'] = true;
			$directives['max-image-preview:large'] = true;
			$directives['max-video-preview:-1'] = true;
		}

		if (empty($directives['nofollow'])) {
			$additionalDirectives['follow'] = true;
		}

		// Ready for IF the additional directives (index, follow) should come into play one day. For now, stick to whatever is default for the search engines
		// return ! empty( $additionalDirectives ) ? array_merge( $additionalDirectives, $directives ) : $directives;

		return $directives;
	}

	public function addAnalytics()
	{
		$headerScripts = null;

		if (is_singular(self::$postTypes)) {
			$post = get_post();
			$headerScripts = get_post_meta($post->ID, '_genesis_scripts', true);
		}

		if ($term = get_queried_object()) {
			if (self::isSupportedTaxonomy($term)) {
				$headerScripts = get_term_meta($term->term_id, '_genesis_scripts', true);
			}
		}

		if (is_author()) {
			$userId = get_the_author_meta('ID');
			$headerScripts = get_user_meta($userId, '_genesis_scripts', true);
		}

		if (!empty($headerScripts)) {
			echo $headerScripts;
		}

		$analyticsScriptAdmin = (bool) get_option('wpsseo_analytics_admin_users', false);
		if (!$analyticsScriptAdmin && is_user_logged_in()) {
			if (current_user_can('manage_options')) {
				return;
			}
		}

		$analyticsScript = get_option('wpsseo_analytics');

		if (!empty($analyticsScript)) {
			echo apply_filters('wpsseo_script_analytics', $analyticsScript);
		}

		return;
	}

	public function addBodyScripts()
	{
		$bodyScripts = null;

		if (is_singular(self::$postTypes)) {
			$post = get_post();
			$bodyScripts = get_post_meta($post->ID, '_genesis_scripts_body', true);
		}

		if ($term = get_queried_object()) {
			if (self::isSupportedTaxonomy($term)) {
				$bodyScripts = get_term_meta($term->term_id, '_genesis_scripts_body', true);
			}
		}

		if (is_author()) {
			$userId = get_the_author_meta('ID');
			$bodyScripts = get_user_meta($userId, '_genesis_scripts_body', true);
		}

		if (!empty($bodyScripts)) {
			echo $bodyScripts;
		}

		$bodyScriptsAdmin = (bool) get_option('wpsseo_body_scripts_admin_users', false);
		if (!$bodyScriptsAdmin && is_user_logged_in()) {
			if (current_user_can('manage_options')) {
				return;
			}
		}

		$bodyScripts = get_option('wpsseo_body_scripts');

		if (!empty($bodyScripts)) {
			echo apply_filters('wpsseo_script_body', $bodyScripts);
		}

		return;
	}
}
