<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftSchema
{
	public $options;

	public function __construct()
	{
		if (is_admin()) {
			return;
		}

		$defaultOptions = get_option('wpsseo_schema_data', []);

		if (empty($defaultOptions)) {
			return;
		}

		$this->options = $defaultOptions;
		$this->options['home_url'] = trailingslashit(home_url());
		$this->options['language'] = str_replace('_', '-', get_locale());
		$this->options['theme_mods'] = get_theme_mods();

		add_filter('wpsseo_meta_tags', [$this, 'addSchema']);
	}

	public static function init()
	{
		return new self();
	}

	private function getPostObject()
	{
		if (SwiftMetaTags::isHome()) {
			$homePageId = get_option('page_for_posts', 0);
			$post = get_post($homePageId);
		} elseif (SwiftMetaTags::isFrontPage()) {
			$frontPageId = get_option('page_on_front', 0);
			$post = get_post($frontPageId);
		} elseif (is_singular() && !is_singular('product')) {
			$post = get_post();
		}

		return $post ?? null;
	}

	// For methods that is used by more than one schema type, we created a private function to handle the logic
	private function getObjectType()
	{
		if (SwiftMetaTags::isFrontPage() || is_singular()) {
			if (!is_singular('product')) {
				return 'post';
			}
		}

		return null;
	}

	private function getSchemaPageType()
	{
		$pageTypes = ['WebPage'];

		if (is_singular(array_keys(SwiftPostTypes::getPostTypes()))) {
			$post = get_post();

			if ($post->post_type === 'post') {
				$pageTypes[] = 'Article';
			}
		}

		return array_unique($pageTypes);
	}

	public function getWebsiteSchema()
	{
		$homeUrl = $this->options['home_url'];
		$frontPage = get_option('show_on_front');
		$frontPageDescription = '';

		if ($frontPage === 'page') {
			$frontPageId = get_option('page_on_front');
			$frontPageDescription = get_post_meta($frontPageId, '_genesis_description', true);
		}

		if (empty($frontPageDescription)) {
			$frontPageDescription = get_option('wpsseo_home_seo_description') ?? get_bloginfo('description');
		}

		$website = [
			'@type' => 'WebSite',
			'@id' => "{$homeUrl}#website",
			'name' => $this->options['company_name'],
			'description' => $frontPageDescription,
			'url' => $homeUrl,
			'inLanguage' => $this->options['language'],
			'potentialAction' => [
				'@type' => 'SearchAction',
				'target' => "{$homeUrl}?s={search_term_string}",
				'query-input' => 'required name=search_term_string',
			],
		];

		$website['publisher'] = [
			'@id' => "{$homeUrl}#organization",
		];

		$website['copyrightHolder'] = [
			'@id' => "{$homeUrl}#organization",
		];

		return $website;
	}

	public function getBreadcrumbsSchema()
	{
		$breadcrumbs = SwiftBreadcrumbs::getBreadcrumbsStructure();
		$schema = [];

		if (!empty($breadcrumbs)) {
			$items		 = [];
			$itemPosition = 1;

			foreach ($breadcrumbs as $crumb) {
				$items[] = [
					'@type' => 'ListItem',
					'position' => $itemPosition++,
					'name' => $crumb['label'],
					'item' => $crumb['url'],
				];
			}

			if (!empty($items)) {
				$schema = [
					'@type' => 'BreadcrumbList',
					'@id' => WPSSEO_CURRENT_URL . '#breadcrumb',
					'itemListElement' => $items,
				];
			}
		}

		return $schema;
	}

	public function getOrganizationSchema()
	{
		$homeUrl = $this->options['home_url'];
		$organizationType = $this->options['organization_type'] ?: 'Organization';
		$organizationType = $organizationType !== 'Organization' ? [$organizationType, 'Organization'] : $organizationType;
		$companySocialProfiles = array_filter((array) $this->options['social_profiles']);

		$organization = [
			'@type' => $organizationType,
			'@id' => "$homeUrl#organization",
			'name' => $this->options['company_name'],
			'url' => $homeUrl,
		];

		if (!empty($this->options['company_vat'])) {
			$organization['vatID'] = $this->options['company_vat'];
		}

		if (!empty($this->options['company_email'])) {
			$organization['email'] = $this->options['company_email'];
		}

		if (!empty($this->options['company_phone'])) {
			$organization['telephone'] = $this->options['company_phone'];
		}

		if (!empty($this->options['company_address']) && !empty($this->options['company_zip']) && !empty($this->options['company_city']) && !empty($this->options['company_country'])) {
			$organization['address'] = [
				'@type' => 'PostalAddress',
				'streetAddress' => $this->options['company_address'],
				'addressLocality' => $this->options['company_city'],
				'postalCode' => $this->options['company_zip'],
				'addressCountry' => $this->options['company_country'],
			];
		}

		$logoId = get_theme_mod('custom_logo');
		if (!empty($logoId)) {
			$logo = wp_get_attachment_image_src($logoId, 'full');

			if ($logo) {
				$organization['logo'] = [
					'@type' => 'ImageObject',
					'inLanguage' => $this->options['language'],
					'@id' => "$homeUrl#/schema/logo/image/",
					'url' => $logo[0],
					'contentUrl' => $logo[0],
					'width' => $logo[1],
					'height' => $logo[2],
					'caption' => $this->options['company_name'],
				];
			}
		}

		if (empty($organization['logo']) && !empty($this->options['theme_mods']['header_image_data'])) {
			$headerImage = $this->options['theme_mods']['header_image_data'];
			$organization['logo'] = [
				'@type' => 'ImageObject',
				'inLanguage' => $this->options['language'],
				'@id' => "$homeUrl#/schema/logo/image/",
				'url' => $headerImage->url,
				'contentUrl' => $headerImage->url,
				'width' => $headerImage->width,
				'height' => $headerImage->height,
				'caption' => $this->options['company_name'],
			];
		}

		if (empty($organization['logo'])) {
			$headerImageUrl = get_header_image();

			if ($headerImageUrl) {
				$headerImageId = attachment_url_to_postid($headerImageUrl);

				if ($headerImageId) {
					$headerImage = wp_get_attachment_image_src($headerImageId, 'full');
					$organization['logo'] = [
						'@type' => 'ImageObject',
						'inLanguage' => $this->options['language'],
						'@id' => "$homeUrl#/schema/logo/image/",
						'url' => $headerImage[0],
						'contentUrl' => $headerImage[0],
						'width' => $headerImage[1],
						'height' => $headerImage[2],
						'caption' => $this->options['company_name'],
					];
				}
			}
		}

		$imageId = get_option('wpsseo_company_image');
		if (!empty($imageId)) {
			$image = wp_get_attachment_image_src($imageId, 'full');

			if ($image) {
				$imageAlt = trim(wp_strip_all_tags(get_post_meta($imageId, '_wp_attachment_image_alt', true)));
				$organization['image'] = [
					'@type' => 'ImageObject',
					'inLanguage' => $this->options['language'],
					'url' => $image[0],
					'contentUrl' => $image[0],
					'width' => $image[1],
					'height' => $image[2],
					'caption' => $imageAlt,
				];
			}
		}

		if (empty($organization['image']) && ! empty($organization['logo'])) {
			$organization['image'] = [
				'@id' => $organization['logo']['@id'],
			];
		}

		if (!empty($companySocialProfiles)) {
			$organization['sameAs'] = array_values($companySocialProfiles);
		}

		$slogan = get_bloginfo('description');
		if (!empty($slogan)) {
			$organization['slogan'] = $slogan;
		}

		if ($organizationType !== 'Organization') {
			$priceRange = get_option('wpsseo_price_range', 0);
			$currenciesAccepted = get_option('wpsseo_currencies_accepted');
			$paymentAccepted = get_option('wpsseo_payment_accepted');

			$organization['priceRange'] = $priceRange;
			if (!empty($currenciesAccepted)) {
				$organization['currenciesAccepted'] = $currenciesAccepted;
			}

			if (!empty($paymentAccepted)) {
				$organization['paymentAccepted'] = $paymentAccepted;
			}
		}

		return $organization;
	}

	public function getNavigationSchema()
	{
		$navigation = $this->options['primary_navigation'];
		$navigationSchema = [];

		if (empty($navigation)) {
			return $navigationSchema;
		}

		$menuItems = wp_get_nav_menu_items($navigation);

		if (!empty($menuItems)) {
			$itemCount = 1;

			foreach ($menuItems as $item) {
				$name = $item->title;
				$url = $item->url;

				if (empty($name) || empty($url)) {
					continue;
				}

				if (strpos($url, 'http') === false) {
					continue;
				}

				$navigationSchema[] = [
					'@type' => 'SiteNavigationElement',
					'position' => $itemCount++,
					'name' => $name,
					'url' => $url,
				];
			}
		}

		return $navigationSchema;
	}

	public function getPersonSchema($post = null)
	{
		$schemaTypes = $this->getSchemaPageType();

		if (empty($post)) {
			$post = $this->getPostObject();
		} else {
			$schemaTypes[] = 'Article';
		}

		if (!in_array('ProfilePage', $schemaTypes) && !in_array('Article', $schemaTypes)) {
			return [];
		}

		$homeUrl = $this->options['home_url'];
		$authorId = $post?->post_author ?? get_the_author_meta('ID');
		$authorMd5 = md5($authorId);
		$authorObject = get_user_by('ID', $authorId);
		$authorSocialProfiles = array_values(SwiftAuthor::getSocialProfiles($authorId));
		$authorName = !empty($authorObject) ? $authorObject->display_name : sprintf(__('User %s', WPSSEO_TEXT_DOMAIN), $authorId);
		$authorUrl = get_author_posts_url($authorId);

		$content = get_user_meta($authorId, 'description', true);
		if (empty($content)) {
			$content = get_user_meta($authorId, 'meta_description', true);
		}

		$authorDescription = SwiftContent::format($content);

		$author = [
			'@type' => 'Person',
			'@id' => "{$homeUrl}#/schema/person/{$authorMd5}",
			'name' => $authorName,
			'description' => $authorDescription,
			'url' => $authorUrl,
			'image' => [
				'@type' => 'ImageObject',
				'@id' => "{$homeUrl}#/schema/person/image/",
				'url' => get_avatar_url($authorObject->user_email, ['size' => 100, 'force_default' => true, 'default' => 'mm']),
				'contentUrl' => get_avatar_url($authorObject->user_email, ['size' => 100, 'force_default' => true, 'default' => 'mm']),
				'caption' => $authorName,
			],
		];

		if (!empty($authorSocialProfiles)) {
			$author['sameAs'] = $authorSocialProfiles;
		}

		return $author;
	}

	public function getPageSchema()
	{
		$type = $this->getObjectType();
		if ($type !== 'post') {
			return [];
		}

		$schemaTypes = $this->getSchemaPageType();
		$articleSchema = array_search('Article', $schemaTypes);

		if ($articleSchema !== false) {
			unset($schemaTypes[$articleSchema]);
		}

		$schemaType = count($schemaTypes) < 2 ? $schemaTypes[0] : array_values($schemaTypes);
		$object = $this->getPostObject();
		$objectId = $object?->ID;

		if (empty($objectId)) {
			return [];
		}

		$pageName = SwiftContent::format($object->post_title);
		$pageDescription = get_post_meta($objectId, '_genesis_description', true);
		$thumbnailUrl = get_the_post_thumbnail_url($objectId, 'full');

		$pageSchema = [
			'@type' => $schemaType,
			'@id' => WPSSEO_CURRENT_URL,
			'url' => WPSSEO_CURRENT_URL,
			'name' => $pageName,
			'isPartOf' => ['@id' => $this->options['home_url'] . '#website'],
			'primaryImageOfPage' => [
				'@id' => WPSSEO_CURRENT_URL . '#primaryimage',
			],
			'image' => [
				'@id' => WPSSEO_CURRENT_URL . '#primaryimage',
			],
			'thumbnailUrl' => $thumbnailUrl,
			'description' => SwiftContent::format($pageDescription),
			'breadcrumb' => [
				'@id' => WPSSEO_CURRENT_URL . '#breadcrumb',
			],
			'inLanguage' => $this->options['language'],
			'potentialAction' => [
				'@type' => 'ReadAction',
				'target' => WPSSEO_CURRENT_URL,
			],
		];

		if (empty($thumbnailUrl)) {
			unset($pageSchema['thumbnailUrl']);
		}

		$pageSchema['datePublished'] = get_the_date('c', $objectId);
		$pageSchema['dateModified'] = get_the_modified_date('c', $objectId);

		return $pageSchema;
	}

	public function getArticleSchema($post = null)
	{
		$schemaTypes = $this->getSchemaPageType();
		$getFullAuthor = false;

		if (empty($post)) {
			$post = $this->getPostObject();
		} else {
			$schemaTypes[] = 'Article';
			$getFullAuthor = true;
		}

		if (empty($post)) {
			return [];
		}

		if (!in_array('Article', $schemaTypes)) {
			return [];
		}

		$author = $this->getPersonSchema($post);
		$authorMarkup = [
			'@id' => $author['@id'],
			'name' => $author['name'],
		];

		if ($getFullAuthor) {
			$authorMarkup = $author;
		}

		$thumbnailUrl = get_the_post_thumbnail_url($post->ID, 'full');
		$postTags = wp_get_post_tags($post->ID, ['fields' => 'names']);
		$publishedDate = get_the_date('c', $post->ID);
		$comments = get_comments([
			'post_id' => $post->ID,
			'status' => 'approve',
			'fields' => 'ids',
		]);

		$articleSchema = [
			'@type' => 'Article',
			'@id' => WPSSEO_CURRENT_URL . '#article-' . $post->ID,
			'isPartOf' => ['@id' => WPSSEO_CURRENT_URL],
			'mainEntityOfPage' => ['@id' => WPSSEO_CURRENT_URL],
			'author' => $authorMarkup,
			'headline' => SwiftContent::format($post->post_title),
			'datePublished' => $publishedDate,
			'dateModified' => get_the_modified_date('c', $post->ID),
			'commentCount' => count($comments),
			'publisher' => ['@id' => $this->options['home_url'] . '#organization'],
			'image' => [
				'@id' => WPSSEO_CURRENT_URL . '#primaryimage',
			],
			'inLanguage' => $this->options['language'],
			'copyrightYear' => substr($publishedDate, 0, 4),
			'copyrightHolder' => [
				'@id' => $this->options['home_url'] . '#organization',
			],

		];

		if (!empty($thumbnailUrl)) {
			$articleSchema['thumbnailUrl'] = $thumbnailUrl;
		}

		if (!empty($postTags)) {
			$articleSchema['keywords'] = $postTags;
		}

		if (comments_open($post->ID)) {
			$articleSchema['potentialAction'] = [
				'@type' => 'CommentAction',
				'name' => 'Comment',
				'target' => WPSSEO_CURRENT_URL . '#respond',
			];
		}

		return $articleSchema;
	}

	public function getImageSchema()
	{
		$postId = 0;
		$pageImageId = 0;
		$pageImage = null;

		if (SwiftMetaTags::isHome()) {
			$postId = get_option('page_for_posts', 0);
			$pageImageId = get_post_thumbnail_id($postId);
		} elseif (SwiftMetaTags::isFrontPage()) {
			$postId = get_option('page_on_front', 0);
			$pageImageId = get_post_thumbnail_id($postId);
		} elseif (is_singular()) {
			$post = get_post();
			$postId = $post->ID;
			$pageImageId = get_post_thumbnail_id($postId);
		}

		if (empty($pageImageId)) {
			if (! empty($postId)) {
				$pageImageId = get_post_meta($postId, '_social_image_id', true);
			}
		}

		if (empty($pageImageId)) {
			$pageImageId = get_option('wpsseo_default_social_image');
		}

		if (!empty($pageImageId)) {
			$pageImage = wp_get_attachment_image_src($pageImageId, 'full');
		}

		if (!empty($pageImage)) {
			return [
				'@type' => 'ImageObject',
				'inLanguage' => $this->options['language'],
				'@id' => WPSSEO_CURRENT_URL . '#primaryimage',
				'url' => $pageImage[0],
				'contentUrl' => $pageImage[0],
				'width' => $pageImage[1],
				'height' => $pageImage[2]
			];
		}
	}

	public function buildSchema()
	{
		$schema = [];

		if (empty($this->options)) {
			return $schema;
		}

		$websiteSchema = $this->getWebsiteSchema();
		$organizationSchema = $this->getOrganizationSchema();
		$navigationSchema = $this->getNavigationSchema();
		$breadcrumbSchema = $this->getBreadcrumbsSchema();
		$imageSchema = $this->getImageSchema();
		$pageSchema = $this->getPageSchema();
		$articleSchema = $this->getArticleSchema();
		$personSchema = $this->getPersonSchema();

		if (empty($breadcrumbSchema)) {
			unset($pageSchema['breadcrumb']);
		}

		$schema = [
			$websiteSchema,
			$organizationSchema,
			...$navigationSchema,
			$breadcrumbSchema,
			$imageSchema,
			$pageSchema,
			$articleSchema,
			$personSchema,
		];

		return array_filter($schema);
	}

	public function addSchema($metaTags)
	{
		$schema = $this->buildSchema();
		if (empty($schema)) {
			return $metaTags;
		}

		$schemaGraph = [
			'@context' => 'https://schema.org',
			'@graph' => array_values($schema),
		];

		if (!empty($schema)) {
			// Keep this for debugging purposes
			// $metaTags .= '<script type="application/ld+json">' . wp_json_encode( $schemaGraph, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE ) . '</script>' . "\n";

			$metaTags .= '<script type="application/ld+json">' . wp_json_encode($schemaGraph) . '</script>' . "\n";
		}

		return $metaTags;
	}
}
