<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftSitemap
{
	public $frontPageId = 0;
	public $postTypes;
	public $taxonomies;
	public $noindexAuto;
	public $noindexPostTypes;
	public $noindexTaxonomies;
	public $noindexArchives;
	public $urlsPerSitemap;

	public function __construct()
	{
		if (is_admin()) {
			return;
		}

		add_filter('wp_sitemaps_enabled', '__return_false');
		add_filter('query_vars', [$this, 'registerSitemapVars']);
		add_action('init', [$this, 'registerProperties'], 999);
		add_action('send_headers', [$this, 'outputSitemap']);
		add_action('send_headers', [$this, 'outputSitemapStylesheet']);
		add_action('template_redirect', [$this, 'maybeRedirectDefaultSitemap'], 5);
	}

	public static function init()
	{
		// Rewrites has to be added directly for consistency
		add_action('init', function () {
			$sitemapFilename = preg_quote(self::getSitemapName());

			add_rewrite_rule('^' . $sitemapFilename . '$', 'index.php?wpsseo-xml-sitemap=true', 'top');
			add_rewrite_rule('^([a-zA-Z_-]+)-sitemap.xml$', 'index.php?wpsseo-xml-sitemap=true&wpsseo-xml-sitemap-type=$matches[1]', 'top');
			add_rewrite_rule('^([a-zA-Z_-]+)-sitemap-([0-9]+).xml$', 'index.php?wpsseo-xml-sitemap=true&wpsseo-xml-sitemap-type=$matches[1]&wpsseo-xml-sitemap-page=$matches[2]', 'top');
			add_rewrite_rule('^sitemap.xsl$', 'index.php?wpsseo-xml-sitemap-stylesheet=true', 'top');
		});

		return new self();
	}

	public function registerProperties()
	{
		$this->postTypes = array_keys(SwiftPostTypes::getPostTypes());
		$this->taxonomies = array_keys(SwiftTaxonomies::getTaxonomies());
		$this->noindexPostTypes = (array) get_option('wpsseo_noindex_post_types');
		$this->noindexTaxonomies = (array) get_option('wpsseo_noindex_taxonomies');
		$this->noindexArchives = (array) get_option('wpsseo_noindex_archives');
		$this->urlsPerSitemap = (int) get_option('wpsseo_sitemap_post_limits', 10000);
		$this->noindexAuto = $this->getNoindexAuto();

		if (get_option('show_on_front', 'posts') === 'page') {
			$this->frontPageId = (int) get_option('page_on_front', 0);
		}
	}

	private function getNoindexAuto()
	{
		global $wpdb;

		$noindexedIds = $wpdb->get_col("SELECT `post_id` FROM `{$wpdb->postmeta}` WHERE `meta_key` = '_genesis_noindex' AND `meta_value` = 1");
		$woocommerceIds = SwiftPostTypes::woocommerceIds();

		if (!empty($woocommerceIds)) {
			$noindexedIds = array_merge($noindexedIds, $woocommerceIds);
		}

		return array_filter($noindexedIds);
	}

	public static function getSitemapName()
	{
		return 'sitemap-index.xml';
	}

	public function maybeRedirectDefaultSitemap()
	{
		$sitemapFilename = self::getSitemapName();
		$currentUrl = htmlspecialchars(filter_input(INPUT_SERVER, 'REQUEST_URI'));

		if ($sitemapFilename !== 'wp-sitemap.xml' && strpos($currentUrl, '/wp-sitemap.xml') !== false) {
			wp_redirect(home_url($sitemapFilename), 301, 'Swift SEO - Redirect Sitemap');
			exit;
		}

		if ($sitemapFilename !== 'sitemap.xml' && strpos($currentUrl, '/sitemap.xml') !== false) {
			wp_redirect(home_url($sitemapFilename), 301, 'Swift SEO - Redirect Sitemap');
			exit;
		}
	}

	public function registerSitemapVars($queryVars)
	{
		$queryVars[] = 'wpsseo-xml-sitemap';
		$queryVars[] = 'wpsseo-xml-sitemap-page';
		$queryVars[] = 'wpsseo-xml-sitemap-type'; //post, page, category, etc.
		$queryVars[] = 'wpsseo-xml-sitemap-stylesheet';

		return $queryVars;
	}

	public function outputSitemap()
	{
		if (empty(get_query_var('wpsseo-xml-sitemap'))) {
			return;
		}

		$type = get_query_var('wpsseo-xml-sitemap-type') ?: 'index';
		$page = (int) get_query_var('wpsseo-xml-sitemap-page') ?: 1;

		$cacheKey = 'wpsseo_sitemap_' . $type . '_' . md5($type . '_' . $page);
		$cachedSitemap = get_transient($cacheKey);

		if ($cachedSitemap) {
			$this->sendXmlHeaders();
			echo $cachedSitemap;
			exit;
		}

		$xmlTags = $this->generateSitemapXml($type, $page);

		set_transient($cacheKey, $xmlTags, DAY_IN_SECONDS);

		$this->sendXmlHeaders();
		echo $xmlTags;
		exit;
	}

	private function sendXmlHeaders($type = 'xml')
	{
		header("Content-Type: text/{$type}");
		header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
		header('Cache-Control: post-check=0, pre-check=0', false);
		header('Pragma: no-cache');
	}

	private function generateSitemapXml($type, $page)
	{
		if ((empty($type) || $type === 'index') && $page === 1) {
			return $this->generateSitemapIndex();
		}

		if (in_array($type, $this->postTypes) && empty($this->noindexPostTypes[$type])) {
			return $this->getPostUrls($type, $page);
		}

		if (in_array($type, $this->taxonomies) && empty($this->noindexTaxonomies[$type])) {
			return $this->getTaxonomyUrls($type);
		}

		if ($type === 'author' && empty($this->noindexArchives['author'])) {
			return $this->getAuthorUrls();
		}

		return $this->wrapUrlSet(''); // Return an empty sitemap if type is excluded
	}

	private function generateSitemapIndex()
	{
		$sitemapUrls = $this->getSitemapIndexUrls();
		$sitemapUrlsXml = '';

		foreach ($sitemapUrls as $url) {
			$sitemapUrlsXml .= "<sitemap><loc>{$url}</loc></sitemap>\n";
		}

		return '<?xml version="1.0" encoding="UTF-8"?>' . "\n"
			. '<?xml-stylesheet type="text/xsl" href="' . home_url('sitemap.xsl') . '"?>' . "\n"
			. '<!-- Sitemap was generated on ' . date('Y-m-d H:i:s') . ' GMT -->' . "\n"
			. '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n"
			. $sitemapUrlsXml
			. '</sitemapindex>';
	}

	private function getPostUrls($postType, int $page)
	{
		global $wpdb;

		$limit = $this->urlsPerSitemap;
		$offset = ($page - 1) * $limit;

		if ($postType === 'page' && empty($this->frontPageId)) {
			if ($page === 1) {
				$limit--;
			} else {
				$offset--;
			}
		}

		$maybeExclude = $this->getExcludePostsClause($wpdb);
		$posts = $wpdb->get_results($wpdb->prepare(
			"SELECT `ID` FROM `{$wpdb->prefix}posts` WHERE `post_type` = %s AND `post_status` = 'publish' {$maybeExclude} LIMIT %d OFFSET %d",
			$postType,
			$limit,
			$offset
		));

		// Check if it's the last page
		$totalPosts = $wpdb->get_var($wpdb->prepare(
			"SELECT COUNT(*) FROM `{$wpdb->prefix}posts` WHERE `post_type` = %s AND `post_status` = 'publish' {$maybeExclude}",
			$postType
		));

		$totalPages = (int) ceil($totalPosts / $this->urlsPerSitemap);
		if ($page === $totalPages) {
			$dynamicUrls = apply_filters("wpsseo_sitemap_{$postType}s", []);
			$posts = array_merge($posts, $dynamicUrls);
		}
		// End check

		$xmlTags = $this->generatePostUrlTags($posts);

		if ($postType === 'page' && $page === 1 && empty($this->frontPageId)) {
			$xmlTags = $this->getFrontPageUrlTag() . $xmlTags;
		}

		return $this->wrapUrlSet($xmlTags);
	}

	private function getExcludePostsClause($wpdb)
	{
		if (empty($this->noindexAuto)) {
			return '';
		}

		$placeholders = implode(',', array_fill(0, count($this->noindexAuto), '%d'));
		return $wpdb->prepare("AND `ID` NOT IN ({$placeholders})", ...$this->noindexAuto);
	}

	private function generatePostUrlTags($posts)
	{
		$xmlTags = '';

		foreach ($posts as $post) {
			if (empty($post->ID)) {
				$xmlTags .= $this->generateDynamicUrlTag($post);
				continue;
			}

			$xmlTags .= $this->generatePostUrlTag($post->ID);
		}

		return $xmlTags;
	}

	private function getPostAttachments($postId)
	{
		global $wpdb;

		return $wpdb->get_results($wpdb->prepare(
			"SELECT `guid` as `url`, `post_mime_type` as `mime` FROM `{$wpdb->prefix}posts` WHERE `post_type` = 'attachment' AND `post_parent` = %d LIMIT 1000",
			$postId
		));
	}

	private function generatePostUrlTag($postId)
	{
		$postTimestamp = get_the_date('U', $postId);
		$postLastmod = date('Y-m-d\TH:i:sP', $postTimestamp);

		$priority = (!empty($this->frontPageId) && $this->frontPageId === (int)$postId) ? '<priority>0.8</priority>' : '';

		$xmlTags = '<url>'
			. '<loc>' . htmlspecialchars(get_permalink($postId)) . '</loc>'
			. '<lastmod>' . $postLastmod . '</lastmod>'
			. $priority;

		$attachments = $this->getPostAttachments($postId);
		if (!empty($attachments)) {
			$xmlTags .= "\n" . $this->generateImageTags($attachments);
		}

		$xmlTags .= '</url>' . "\n";

		return $xmlTags;
	}

	private function generateDynamicUrlTag($post)
	{
		if (!empty($post->url)) {
			return '<url>'
				. '<loc>' . htmlspecialchars((string) $post->url) . '</loc>'
				. '</url>' . "\n";
		}

		return '';
	}

	private function generateImageTags($attachments)
	{
		$xmlTags = '';

		foreach ($attachments as $attachment) {
			if (stripos($attachment->mime, 'image/') !== false) {
				$xmlTags .= '<image:image>'
					. '<image:loc>' . $attachment->url . '</image:loc>'
					. '</image:image>' . "\n";
			}
		}

		return $xmlTags;
	}

	private function getFrontPageUrlTag()
	{
		return '<url>'
			. '<loc>' . htmlspecialchars(home_url('/')) . '</loc>'
			. '<priority>0.8</priority>'
			. '</url>' . "\n";
	}

	private function getTaxonomyUrls($taxonomy)
	{
		$exclude = $this->getExcludeTerms();
		$terms = get_terms([
			'taxonomy' => $taxonomy,
			'hide_empty' => (bool)get_option('wpsseo_noindex_empty_terms', true),
			'exclude' => $exclude,
		]);

		$xmlTags = '';

		foreach ($terms as $term) {
			$xmlTags .= '<url>'
				. '<loc>' . htmlspecialchars(get_term_link($term)) . '</loc>'
				. '</url>' . "\n";
		}

		return $this->wrapUrlSet($xmlTags);
	}

	private function getExcludeTerms()
	{
		global $wpdb;

		$noindexTerms = $wpdb->get_col("SELECT `term_id` FROM `{$wpdb->termmeta}` WHERE `meta_key` = 'noindex' AND `meta_value` = 1");

		if (empty($noindexTerms)) {
			return [];
		}

		return $noindexTerms;
	}

	private function getAuthorUrls()
	{
		global $wpdb;

		$maybeExclude = $this->getExcludeAuthorsClause($wpdb);
		$posts = $wpdb->get_results("SELECT DISTINCT `post_author` FROM `{$wpdb->prefix}posts` WHERE `post_status` = 'publish' AND NOT `post_author` = 0 {$maybeExclude}");

		$xmlTags = '';

		foreach ($posts as $post) {
			$xmlTags .= '<url>'
				. '<loc>' . htmlspecialchars(get_author_posts_url($post->post_author)) . '</loc>'
				. '</url>';
		}

		return $this->wrapUrlSet($xmlTags);
	}

	private function getExcludeAuthorsClause($wpdb)
	{
		$noindexedIds = $wpdb->get_col("SELECT `user_id` FROM `{$wpdb->usermeta}` WHERE `meta_key` = 'noindex' AND `meta_value` = 1");

		if (empty($noindexedIds)) {
			return '';
		}

		$placeholders = implode(',', array_fill(0, count($noindexedIds), '%d'));
		return $wpdb->prepare("AND `post_author` NOT IN ({$placeholders})", ...$noindexedIds);
	}

	private function wrapUrlSet($xmlTags)
	{
		return '<?xml version="1.0" encoding="UTF-8"?>' . "\n"
			. '<?xml-stylesheet type="text/xsl" href="' . home_url('sitemap.xsl') . '"?>' . "\n"
			. '<!-- Sitemap was generated on ' . date('Y-m-d H:i:s') . ' GMT -->' . "\n"
			. '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" '
			. 'xmlns:image="http://www.google.com/schemas/sitemap-image/1.1">' . "\n"
			. $xmlTags
			. '</urlset>';
	}

	private function getSitemapIndexUrls()
	{
		$sitemapUrls = [];

		foreach ($this->postTypes as $postType) {
			if (!empty($this->noindexPostTypes[$postType])) {
				continue;
			}

			$totalPosts = wp_count_posts($postType)->publish;

			if ($totalPosts > 0) {
				$numberOfSitemaps = ceil($totalPosts / $this->urlsPerSitemap);

				if ($numberOfSitemaps > 1) {
					for ($i = 1; $i <= $numberOfSitemaps; $i++) {
						$sitemapUrls[] = home_url("{$postType}-sitemap-{$i}.xml");
					}
				} else {
					$sitemapUrls[] = home_url("{$postType}-sitemap.xml");
				}
			}
		}

		foreach ($this->taxonomies as $taxonomy) {
			if (!empty($this->noindexTaxonomies[$taxonomy])) {
				continue;
			}

			$terms = get_terms([
				'taxonomy' => $taxonomy,
				'hide_empty' => (bool)get_option('wpsseo_noindex_empty_terms', true),
			]);

			if (!empty($terms)) {
				$sitemapUrls[] = home_url("{$taxonomy}-sitemap.xml");
			}
		}

		if (empty($this->noindexArchives['author'])) {
			$sitemapUrls[] = home_url("author-sitemap.xml");
		}

		return $sitemapUrls;
	}

	public function outputSitemapStylesheet()
	{
		if (empty(get_query_var('wpsseo-xml-sitemap-stylesheet'))) {
			return;
		}

		$cacheKey = 'wpsseo_sitemap_stylesheet';
		$xmlStylesheet = get_transient($cacheKey);

		if ($xmlStylesheet) {
			$this->sendXmlHeaders('xsl');
			echo $xmlStylesheet;
			exit;
		}

		$variables  = '<xsl:variable name="tableMinWidth" select="\'700\'"/>';
		$variables .= '<xsl:variable name="colorMain" select="\'#000\'"/>';
		$variables .= '<xsl:variable name="colorAccent" select="\'' . apply_filters('wpsseo_sitemap_accent_color', '#c52f21') . '\'"/>';
		$variables .= '<xsl:variable name="relativeFontColor" select="\'#ffffff\'"/>';

		$iconUrl = get_site_icon_url(32, 'https://swiftseo.app/wp-content/uploads/cropped-swift-seo-icon-32x32.png');
		$icon = '<link rel="icon" href="' . $iconUrl . '" sizes="32x32"/>';
		$generatorLink = '<a href="https://swiftseo.app/" target="_blank" rel="nofollow noopener">Swift SEO</a>';

		$xmlStylesheet = '<?xml version="1.0" encoding="UTF-8"?><xsl:stylesheet version="2.0" xmlns:image="http://www.google.com/schemas/sitemap-image/1.1" xmlns:sitemap="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"><xsl:output method="html" version="1.0" encoding="UTF-8" indent="yes"/><xsl:template match="/"><html xmlns="http://www.w3.org/1999/xhtml">
			<head><meta http-equiv="Content-Type" content="text/html; charset=utf-8"/><meta name="viewport" content="width=device-width, initial-scale=1"/>' . $variables . '<title>XML Sitemap &#x2d; ' . get_option('blogname') . '</title><style style="text/css">html{font-size: 62.5%;height: 100%;}body{font-size: 1.4rem;font-family: -apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,Oxygen-Sans,Ubuntu,Cantarell,"Helvetica Neue",sans-serif;min-height: 100%;display: grid;grid-template-rows: auto 1fr auto;margin: 0;}.wrap{max-width: <xsl:value-of select="concat( $tableMinWidth, \'px\' )"/>;margin: 0 auto;overflow-wrap: break-word;}a{color: #373c44;text-decoration: none;}h1{font-size: 2.4rem;font-family: Verdana,Geneva,sans-serif;font-weight: normal;margin: 0;color: <xsl:value-of select="$colorAccent"/>;}h1 img{vertical-align: bottom;margin-right: 1.4rem;image-rendering: -webkit-optimize-contrast;}.rtl h1 img{margin-right: unset;margin-left: 1.4rem;}#description{background-color: <xsl:value-of select="$colorMain"/>;border-bottom: .7rem solid <xsl:value-of select="$colorAccent"/>;color: <xsl:value-of select="$relativeFontColor"/>;padding: 2rem 2rem 1.3rem;}#description a{color: <xsl:value-of select="$relativeFontColor"/>;}#content{padding: 2rem;background: #fff;}a:hover{border-bottom: 1px solid;}table{border-spacing: 0;table-layout: fixed;}th, td{font-size: 1.2rem;border: 0px solid;padding: 1rem 1.5rem;width: 100%;max-width: <xsl:value-of select="concat( $tableMinWidth - 159, \'px\' )"/>;min-width: 99px;overflow-wrap: anywhere;}th{text-align: left;border-bottom: 1px solid <xsl:value-of select="$colorAccent"/>;}.rtl th{text-align: right;}tr:nth-of-type(2n){background-color: #eaeaea;}#footer{padding: 0 3rem 2rem;font-size: 1.1rem;color: #999;}#footer a{color: inherit;}#description a, #footer a{border-bottom: 1px solid;}#description a:hover, #footer a:hover{border-bottom: none;}</style>' . $icon . '</head>
			<body class="ltr">
				<div id="description">
					<div class="wrap">
						<a href="' . home_url() . '"><h1>' . get_option('blogname') . ' — XML Sitemap</h1></a>
						<p>This is an auto generated XML Sitemap, meant to be used by search engines like <a href="https://www.google.com/" target="_blank" rel="nofollow noreferrer noopener">Google</a> or <a href="https://www.bing.com/" target="_blank" rel="nofollow noreferrer noopener">Bing</a>.</p>
					</div>
				</div>
				<div id="content">
					<div class="wrap">
						<xsl:if test="count(sitemap:sitemapindex/sitemap:sitemap) &gt; 0">
						<p class="expl">
							This XML Sitemap Index file contains <xsl:value-of select="count(sitemap:sitemapindex/sitemap:sitemap)"/> sitemaps.
						</p>
						<table id="sitemap" cellpadding="3">
							<thead>
							<tr>
								<th>Sitemap</th>
								<th>Last Modified</th>
							</tr>
							</thead>
							<tbody>
							<xsl:for-each select="sitemap:sitemapindex/sitemap:sitemap">
								<xsl:variable name="sitemapUrl">
									<xsl:value-of select="sitemap:loc"/>
								</xsl:variable>
								<tr>
									<td>
										<a href="{$sitemapUrl}">
											<xsl:choose>
												<xsl:when test="string-length($sitemapUrl)&gt;95"><xsl:value-of select="substring($sitemapUrl,0,93)"/>...</xsl:when>
												<xsl:otherwise><xsl:value-of select="$sitemapUrl"/></xsl:otherwise>
											</xsl:choose>
										</a>
									</td>
									<td>
										<xsl:value-of select="concat(substring(sitemap:lastmod,0,11),concat(\' \',substring(sitemap:lastmod,12,5)))"/>
									</td>
								</tr>
							</xsl:for-each>
							</tbody>
						</table>
					</xsl:if>
					<xsl:if test="count(sitemap:sitemapindex/sitemap:sitemap) &lt; 1">
						<p class="expl">
							This XML Sitemap contains <xsl:value-of select="count(sitemap:urlset/sitemap:url)"/> URLs.
						</p>
						<table id="sitemap" cellpadding="3">
							<thead>
							<tr>
								<th>URL</th>
								<th>Images</th>
								<th>Last Modified</th>
							</tr>
							</thead>
							<tbody>
								<xsl:for-each select="sitemap:urlset/sitemap:url">
									<tr>
										<td>
											<xsl:variable name="itemUrl">
												<xsl:value-of select="sitemap:loc"/>
											</xsl:variable>
											<a href="{$itemUrl}">
												<xsl:choose>
													<xsl:when test="string-length($itemUrl)&gt;95"><xsl:value-of select="substring($itemUrl,0,93)"/>...</xsl:when>
													<xsl:otherwise><xsl:value-of select="$itemUrl"/></xsl:otherwise>
												</xsl:choose>
											</a>
										</td>
										<td>
											<xsl:value-of select="count(image:image)"/>
										</td>
										<td>
											<xsl:value-of select="concat(substring(sitemap:lastmod,0,11),concat(\' \',substring(sitemap:lastmod,12,5)))"/>
										</td>
									</tr>
								</xsl:for-each>
							</tbody>
						</table>
					</xsl:if>
				</div>
			</div>
			<div id="footer">
				<div class="wrap"><p>Generated by ' . $generatorLink . '</p></div>
			</div>
		</body>
		</html></xsl:template></xsl:stylesheet>';

		set_transient($cacheKey, $xmlStylesheet, MONTH_IN_SECONDS);

		$this->sendXmlHeaders('xsl');
		echo $xmlStylesheet;
		exit;
	}
}
