<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftUpdater
{
	public $pluginSlug;
	public $version;
	private $cacheKey = 'wpsseo_plugin_update';
	private $cacheAllowed = true;
	private $forceCheck = true;

	public function __construct()
	{
		if (!is_admin()) {
			return;
		}

		$forceCheckUpdates = filter_input(INPUT_GET, 'force-check');
		if (!empty($forceCheckUpdates)) {
			$this->forceCheck = true;
			$this->cacheAllowed = false;
		}

		add_filter('plugins_api', [$this, 'info'], 20, 3);
		add_filter('site_transient_update_plugins', [$this, 'update']);
		add_action('upgrader_process_complete', [$this, 'purge'], 10, 2);
	}

	public static function init()
	{
		return new self();
	}

	public function request($transient = null)
	{
		if (!$this->forceCheck) {
			$lastCheck = get_transient($this->cacheKey . '_last_check');
			if ($lastCheck !== false && $lastCheck > strtotime('-12 hours')) {
				$this->cacheAllowed = false;
			}
		}

		$remote = get_transient($this->cacheKey);

		if (false === $remote || !$this->cacheAllowed) {
			$remote = wp_remote_get(
				WPSSEO_UPDATE_URL,
				[
					'timeout' => 10,
					'headers' => [
						'Accept' => 'application/json'
					]
				]
			);

			if (is_wp_error($remote) || 200 !== wp_remote_retrieve_response_code($remote) || empty(wp_remote_retrieve_body($remote))) {
				return $transient ?: false;
			}

			set_transient($this->cacheKey, $remote, DAY_IN_SECONDS);
			set_transient($this->cacheKey . '_last_check', time(), DAY_IN_SECONDS);
		}

		return json_decode(wp_remote_retrieve_body($remote));
	}

	public function info($res, $action, $args)
	{
		if ('plugin_information' !== $action || WPSSEO_SLUG !== $args->slug) {
			return $res;
		}

		$remote = $this->request();
		if (!$remote) {
			return $res;
		}

		return $this->buildPluginInfo($remote);
	}

	private function buildPluginInfo($remote)
	{
		$res = new stdClass();
		$res->name = $remote->name;
		$res->slug = $remote->slug;
		$res->version = $remote->version;
		$res->tested = $remote->tested;
		$res->requires = $remote->requires;
		$res->author = $remote->author;
		$res->author_profile = $remote->author_profile;
		$res->download_link = $remote->download_url;
		$res->trunk = $remote->download_url;
		$res->requires_php = $remote->requires_php;
		$res->last_updated = $remote->last_updated;
		$res->rating = $remote->rating;
		$res->num_ratings = $remote->num_ratings;
		$res->active_installs = ceil($remote->active_installs / 100) * 100;
		$res->sections = [
			'description' => $remote->sections->description,
			'installation' => $remote->sections->installation,
			'changelog' => $remote->sections->changelog
		];

		if (!empty($remote->banners)) {
			$res->banners = (array) $remote->banners;
		}

		if (!empty($remote->icons)) {
			$res->icons = (array) $remote->icons;
		}

		return $res;
	}

	public function update($transient)
	{
		if (empty($transient->checked)) {
			return $transient;
		}

		$remote = $this->request($transient);

		if (
			$remote && version_compare(WPSSEO_VERSION, $remote->version, '<')
			&& version_compare($remote->requires, get_bloginfo('version'), '<=')
			&& version_compare($remote->requires_php, PHP_VERSION, '<')
		) {
			$transient->response[plugin_basename(WPSSEO_FILE)] = $this->prepareUpdateObject($remote);
		}

		return $transient;
	}

	private function prepareUpdateObject($remote)
	{
		$res = new stdClass();
		$res->slug = WPSSEO_SLUG;
		$res->plugin = plugin_basename(WPSSEO_FILE);
		$res->new_version = $remote->version;
		$res->tested = $remote->tested;
		$res->package = $remote->download_url;
		return $res;
	}

	public function purge($upgrader, $options)
	{
		if ($this->cacheAllowed && 'update' === $options['action'] && 'plugin' === $options['type']) {
			delete_transient($this->cacheKey);
		}
	}
}
