<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class SwiftWpMods
{
	public function __construct()
	{
		if (is_admin()) {
			add_action('updated_option', [__CLASS__, 'updateMultisiteRegistration']);
		} else {
			$this->setupFrontendHooks();
		}
	}

	private function setupFrontendHooks()
	{
		add_action('language_attributes', [$this, 'removeCountryCode'], 10, 2);
		add_action('init', [$this, 'frontendInit']);
	}

	public function frontendInit()
	{
		if (is_admin()) {
			return;
		}

		$this->removeGeneratorTags();
		$this->removeRssFeeds();
		$this->disableHeartbeat();
		$this->removeShortlink();
		$this->disableXmlrpc();
	}

	public static function init()
	{
		return new self();
	}

	public static function updateMultisiteRegistration()
	{
		$isMultilanguage = self::checkMultilanguage();
		update_option('wpsseo_is_multilanguage', $isMultilanguage);
		return $isMultilanguage;
	}

	private static function checkMultilanguage()
	{
		if (is_multisite()) {
			$languages = self::getMultisiteLanguages();
			if (count($languages) > 1) {
				return true;
			}
		}

		return self::checkMultilingualPlugins();
	}

	private static function getMultisiteLanguages()
	{
		$languageCodes = wp_cache_get('wpsseo_multisite_languages');
		if (empty($languageCodes)) {
			$languageCodes = array_unique(array_filter(array_map(function ($blog) {
				switch_to_blog($blog->blog_id);
				$lang = get_option('WPLANG', 'en_US');
				restore_current_blog();
				return $lang;
			}, get_sites())));

			wp_cache_set('wpsseo_multisite_languages', $languageCodes, '', 3600);
		}

		return $languageCodes;
	}

	private static function checkMultilingualPlugins()
	{
		return defined('WEGLOT_BNAME') || defined('POLYLANG_BASENAME') ||
			defined('WPML_PLUGIN_BASENAME') || defined('TRP_PLUGIN_BASE') ||
			class_exists('Multilingual_Press');
	}

	public static function isMultilanguage()
	{
		return (bool) get_option('wpsseo_is_multilanguage', false);
	}

	public function removeCountryCode($output, $doctype)
	{
		if (self::isMultilanguage() || !$doctype === 'html') {
			return $output;
		}

		$removeCountryCode = (bool) get_option('wpsseo_remove_country_code', true);
		if (!$removeCountryCode) {
			return $output;
		}

		return self::simplifyLanguageAttribute($output);
	}

	public static function simplifyLanguageAttribute($output)
	{
		$langStr = get_locale() ?: get_bloginfo('language') ?: 'en_US';
		if (strpos($langStr, '_') !== false) {
			list($langCode) = explode('_', $langStr);
			return str_replace($langStr, $langCode, $output);
		}

		return $output;
	}

	public function removeGeneratorTags()
	{
		if (get_option('wpsseo_remove_generator_tags', false)) {
			remove_action('wp_head', 'wp_generator');
			add_filter('the_generator', '__return_empty_string');
		}
	}

	public function removeRssFeeds()
	{
		if (!get_option('wpsseo_remove_rss_feeds', false)) {
			return;
		}

		add_action('do_feed', [$this, 'redirectFeeds'], 1);
		remove_action('wp_head', 'feed_links_extra', 3);
		remove_action('wp_head', 'feed_links', 2);
		remove_action('wp_head', 'rsd_link');
	}

	public function redirectFeeds()
	{
		wp_redirect(home_url(), 301, 'Swift SEO - Redirect Feeds');
		exit;
	}

	public function disableHeartbeat()
	{
		wp_deregister_script('heartbeat');
	}

	public function removeShortlink()
	{
		if (get_option('wpsseo_remove_shortlink', true)) {
			remove_action('wp_head', 'wp_shortlink_wp_head');
		}
	}

	public function disableXmlrpc()
	{
		if (get_option('wpsseo_disable_xmlrpc', true)) {
			add_filter('xmlrpc_enabled', '__return_false');
		}
	}
}
